@@ -1,14 +1,13 @@
 /////////////////////////////////////////////////////////////////////////////
-
 // Name:        class_drawsheet.cpp
-// Purpose:		member functions for DrawSheetStruct
-//				header = class_drawsheet.h
+// Purpose:     member functions for SCH_SHEET
+//              header = class_drawsheet.h
 // Author:      jean-pierre Charras
 // Modified by:
 // Created:     08/02/2006 18:37:02
 // RCS-ID:
 // Copyright:
-// Licence:     License GNU
+// License:     License GNU
 /////////////////////////////////////////////////////////////////////////////
 
 #include "fctsys.h"
@@ -19,21 +18,18 @@
 #include "confirm.h"
 
 #include "program.h"
-#include "libcmp.h"
 #include "general.h"
 #include "protos.h"
 
 
-/***********************************************************/
-DrawSheetStruct::DrawSheetStruct( const wxPoint& pos ) :
+SCH_SHEET::SCH_SHEET( const wxPoint& pos ) :
     SCH_ITEM( NULL, DRAW_SHEET_STRUCT_TYPE )
-/***********************************************************/
 {
-    m_Label            = NULL;
-    m_NbLabel          = 0;
-    m_Layer            = LAYER_SHEET;
-    m_Pos              = pos;
-    m_TimeStamp        = GetTimeStamp();
+    m_Label = NULL;
+    m_NbLabel = 0;
+    m_Layer = LAYER_SHEET;
+    m_Pos = pos;
+    m_TimeStamp = GetTimeStamp();
     m_SheetNameSize    = m_FileNameSize = 60;
     m_AssociatedScreen = NULL;
     m_SheetName.Printf( wxT( "Sheet%8.8lX" ), m_TimeStamp );
@@ -41,11 +37,9 @@ DrawSheetStruct::DrawSheetStruct( const wxPoint& pos ) :
 }
 
 
-/**************************************/
-DrawSheetStruct::~DrawSheetStruct()
-/**************************************/
+SCH_SHEET::~SCH_SHEET()
 {
-    Hierarchical_PIN_Sheet_Struct* label = m_Label, * next_label;
+    SCH_SHEET_PIN* label = m_Label, * next_label;
 
     while( label )
     {
@@ -54,8 +48,8 @@ DrawSheetStruct::~DrawSheetStruct()
         label = next_label;
     }
 
-    //also, look at the associated sheet & its reference count
-    //perhaps it should be deleted also.
+    // also, look at the associated sheet & its reference count
+    // perhaps it should be deleted also.
     if( m_AssociatedScreen )
     {
         m_AssociatedScreen->m_RefCount--;
@@ -65,55 +59,40 @@ DrawSheetStruct::~DrawSheetStruct()
 }
 
 
-/**********************************************/
-bool DrawSheetStruct::Save( FILE* aFile ) const
-/***********************************************/
-
 /** Function Save
  * writes the data structures for this object out to a FILE in "*.brd" format.
  * @param aFile The FILE to write to.
  * @return bool - true if success writing else false.
  */
+bool SCH_SHEET::Save( FILE* aFile ) const
 {
-    bool Success = true;
-    Hierarchical_PIN_Sheet_Struct* SheetLabel;
+    SCH_SHEET_PIN* SheetLabel;
 
-    fprintf( aFile, "$Sheet\n" );
+    if( fprintf( aFile, "$Sheet\n" ) == EOF
+        || fprintf( aFile, "S %-4d %-4d %-4d %-4d\n",
+                    m_Pos.x, m_Pos.y, m_Size.x, m_Size.y ) == EOF )
+        return false;
 
-    if( fprintf( aFile, "S %-4d %-4d %-4d %-4d\n",
-                 m_Pos.x, m_Pos.y,
-                 m_Size.x, m_Size.y ) == EOF )
-    {
-        Success = false;
-        return Success;
-    }
+    //save the unique timestamp, like other schematic parts.
+    if( fprintf( aFile, "U %8.8lX\n", m_TimeStamp ) == EOF )
+        return false;
 
-    //save the unique timestamp, like other shematic parts.
-    if( fprintf( aFile, "U %8.8lX\n", m_TimeStamp )  == EOF )
-    {
-        Success = false; return Success;
-    }
-
-    /* Generation de la liste des 2 textes (sheetname et filename) */
+    /* Save schematic sheetname and filename. */
     if( !m_SheetName.IsEmpty() )
     {
         if( fprintf( aFile, "F0 \"%s\" %d\n", CONV_TO_UTF8( m_SheetName ),
                      m_SheetNameSize ) == EOF )
-        {
-            Success = false; return Success;
-        }
+            return false;
     }
 
     if( !m_FileName.IsEmpty() )
     {
         if( fprintf( aFile, "F1 \"%s\" %d\n", CONV_TO_UTF8( m_FileName ),
                      m_FileNameSize ) == EOF )
-        {
-            Success = false; return Success;
-        }
+            return false;
     }
 
-    /* Generation de la liste des labels (entrees) de la sous feuille */
+    /* Create the list of labels in the sheet. */
     SheetLabel = m_Label;
     int l_id = 2;
     while( SheetLabel != NULL )
@@ -124,34 +103,38 @@ bool DrawSheetStruct::Save( FILE* aFile ) const
         SheetLabel = SheetLabel->Next();
     }
 
-    fprintf( aFile, "$EndSheet\n" );
-    return Success;
+    if( fprintf( aFile, "$EndSheet\n" ) == EOF )
+        return false;
+
+    return true;
 }
 
 
-/***********************************************/
-DrawSheetStruct* DrawSheetStruct::GenCopy()
-/***********************************************/
-
 /* creates a copy of a sheet
  *  The linked data itself (EEDrawList) is not duplicated
  */
+SCH_SHEET* SCH_SHEET::GenCopy()
 {
-    DrawSheetStruct* newitem = new DrawSheetStruct( m_Pos );
+    SCH_SHEET* newitem = new SCH_SHEET( m_Pos );
 
 
     newitem->m_Size = m_Size;
     newitem->SetParent( m_Parent );
     newitem->m_TimeStamp = GetTimeStamp();
 
-    newitem->m_FileName      = m_FileName;
-    newitem->m_FileNameSize  = m_FileNameSize;
-    newitem->m_SheetName     = m_SheetName;
+    newitem->m_FileName     = m_FileName;
+    newitem->m_FileNameSize = m_FileNameSize;
+
+/*    newitem->m_SheetName     = m_SheetName;   m_SheetName must be unique for
+ * all sub sheets in a given sheet
+ *                                               so we no not duplicate sheet
+ * name
+ */
     newitem->m_SheetNameSize = m_SheetNameSize;
 
     newitem->m_Label = NULL;
 
-    Hierarchical_PIN_Sheet_Struct* Slabel = NULL, * label = m_Label;
+    SCH_SHEET_PIN* Slabel = NULL, * label = m_Label;
 
     if( label )
     {
@@ -177,13 +160,10 @@ DrawSheetStruct* DrawSheetStruct::GenCopy()
 }
 
 
-/**********************************************************/
-void DrawSheetStruct::SwapData( DrawSheetStruct* copyitem )
-/**********************************************************/
-
 /* Used if undo / redo command:
  *  swap data between this and copyitem
  */
+void SCH_SHEET::SwapData( SCH_SHEET* copyitem )
 {
     EXCHG( m_Pos, copyitem->m_Pos );
     EXCHG( m_Size, copyitem->m_Size );
@@ -193,8 +173,9 @@ void DrawSheetStruct::SwapData( DrawSheetStruct* copyitem )
     EXCHG( m_Label, copyitem->m_Label );
     EXCHG( m_NbLabel, copyitem->m_NbLabel );
 
-    // Ensure sheet labels have their .m_Parent member poiuntin really on their parent, after swapping.
-    Hierarchical_PIN_Sheet_Struct* label = m_Label;
+    // Ensure sheet labels have their .m_Parent member pointing really on their
+    // parent, after swapping.
+    SCH_SHEET_PIN* label = m_Label;
     while( label )
     {
         label->SetParent( this );
@@ -210,12 +191,11 @@ void DrawSheetStruct::SwapData( DrawSheetStruct* copyitem )
 }
 
 
-/********************************************************************/
-void DrawSheetStruct::Place( WinEDA_SchematicFrame* frame, wxDC* DC )
-/********************************************************************/
+void SCH_SHEET::Place( WinEDA_SchematicFrame* frame, wxDC* DC )
 {
-    /* Placement en liste des structures si nouveau composant:*/
-    bool isnew = (m_Flags & IS_NEW) ? true : false;
+    /* Place list structures for new sheet. */
+    bool isnew = ( m_Flags & IS_NEW ) ? true : false;
+
     if( isnew )
     {
         if( !frame->EditSheet( this, DC ) )
@@ -230,24 +210,22 @@ void DrawSheetStruct::Place( WinEDA_SchematicFrame* frame, wxDC* DC )
     }
 
     SCH_ITEM::Place( frame, DC ); //puts it on the EEDrawList.
-    if ( isnew )
+    if( isnew )
     {
         frame->SetSheetNumberAndCount();
     }
 }
 
 
-/********************************************************************/
-void DrawSheetStruct::CleanupSheet( WinEDA_SchematicFrame* aFrame, bool aRedraw )
-/********************************************************************/
-
 /** Function CleanupSheet
- * Delete pinsheets which are not corresponding to a hierarchal label
+ * Delete pinsheets which are not corresponding to a hierarchical label
  * @param aRedraw = true to redraw Sheet
  * @param aFrame = the schematic frame
  */
+void SCH_SHEET::CleanupSheet( WinEDA_SchematicFrame* aFrame,
+                              bool                   aRedraw )
 {
-    Hierarchical_PIN_Sheet_Struct* Pinsheet, * NextPinsheet;
+    SCH_SHEET_PIN* Pinsheet, * NextPinsheet;
 
     if( !IsOK( aFrame, _( "Ok to cleanup this sheet" ) ) )
         return;
@@ -266,7 +244,7 @@ void DrawSheetStruct::CleanupSheet( WinEDA_SchematicFrame* aFrame, bool aRedraw
 
             HLabel = (SCH_HIERLABEL*) DrawStruct;
             if( Pinsheet->m_Text.CmpNoCase( HLabel->m_Text ) == 0 )
-                break; // Found!
+                break;  // Found!
 
             HLabel = NULL;
         }
@@ -274,7 +252,7 @@ void DrawSheetStruct::CleanupSheet( WinEDA_SchematicFrame* aFrame, bool aRedraw
         NextPinsheet = Pinsheet->Next();
         if( HLabel == NULL )   // Hlabel not found: delete pinsheet
         {
-            aFrame->GetScreen()->SetModify();
+            aFrame->OnModify( );
             aFrame->DeleteSheetLabel( false, Pinsheet );
         }
         Pinsheet = NextPinsheet;
@@ -285,11 +263,14 @@ void DrawSheetStruct::CleanupSheet( WinEDA_SchematicFrame* aFrame, bool aRedraw
 }
 
 
-/**************************************************************************************/
-void DrawSheetStruct::Draw( WinEDA_DrawPanel* aPanel, wxDC* aDC,
-                            const wxPoint& aOffset,
-                            int aDrawMode, int aColor )
-/**************************************************************************************/
+/** Function GetPenSize
+ * @return the size of the "pen" that be used to draw or plot this item
+ */
+int SCH_SHEET::GetPenSize()
+{
+    return g_DrawDefaultLineThickness;
+}
+
 
 /** Function Draw
  *  Draw the hierarchical sheet shape
@@ -297,15 +278,18 @@ void DrawSheetStruct::Draw( WinEDA_DrawPanel* aPanel, wxDC* aDC,
  *  @param aDc = the current Device Context
  *  @param aOffset = draw offset (usually wxPoint(0,0))
  *  @param aDrawMode = draw mode
- *  @param aColor = color used to draw sheet. Usually -1 to use the normal color for sheet items
+ *  @param aColor = color used to draw sheet. Usually -1 to use the normal
+ * color for sheet items
  */
+void SCH_SHEET::Draw( WinEDA_DrawPanel* aPanel, wxDC* aDC,
+                      const wxPoint& aOffset, int aDrawMode, int aColor )
 {
-    Hierarchical_PIN_Sheet_Struct* SheetLabelStruct;
+    SCH_SHEET_PIN* SheetLabelStruct;
     int      txtcolor;
     wxString Text;
     int      color;
     wxPoint  pos = m_Pos + aOffset;
-    int      LineWidth = g_DrawMinimunLineWidth;
+    int      LineWidth = g_DrawDefaultLineThickness;
 
     if( aColor >= 0 )
         color = aColor;
@@ -323,10 +307,11 @@ void DrawSheetStruct::Draw( WinEDA_DrawPanel* aPanel, wxDC* aDC,
         txtcolor = ReturnLayerColor( LAYER_SHEETNAME );
 
     Text = wxT( "Sheet: " ) + m_SheetName;
-    DrawGraphicText( aPanel, aDC,
-                     wxPoint( pos.x, pos.y - 8 ), (EDA_Colors) txtcolor,
-                     Text, TEXT_ORIENT_HORIZ, wxSize( m_SheetNameSize, m_SheetNameSize ),
-                     GR_TEXT_HJUSTIFY_LEFT, GR_TEXT_VJUSTIFY_BOTTOM, LineWidth );
+    DrawGraphicText( aPanel, aDC, wxPoint( pos.x, pos.y - 8 ),
+                     (EDA_Colors) txtcolor, Text, TEXT_ORIENT_HORIZ,
+                     wxSize( m_SheetNameSize, m_SheetNameSize ),
+                     GR_TEXT_HJUSTIFY_LEFT, GR_TEXT_VJUSTIFY_BOTTOM, LineWidth,
+                     false, false, false );
 
     /* Draw text : FileName */
     if( aColor >= 0 )
@@ -334,11 +319,11 @@ void DrawSheetStruct::Draw( WinEDA_DrawPanel* aPanel, wxDC* aDC,
     else
         txtcolor = ReturnLayerColor( LAYER_SHEETFILENAME );
     Text = wxT( "File: " ) + m_FileName;
-    DrawGraphicText( aPanel, aDC,
-                     wxPoint( pos.x, pos.y + m_Size.y + 4 ),
-                     (EDA_Colors) txtcolor,
-                     Text, TEXT_ORIENT_HORIZ, wxSize( m_FileNameSize, m_FileNameSize ),
-                     GR_TEXT_HJUSTIFY_LEFT, GR_TEXT_VJUSTIFY_TOP, LineWidth );
+    DrawGraphicText( aPanel, aDC, wxPoint( pos.x, pos.y + m_Size.y + 4 ),
+                     (EDA_Colors) txtcolor, Text, TEXT_ORIENT_HORIZ,
+                     wxSize( m_FileNameSize, m_FileNameSize ),
+                     GR_TEXT_HJUSTIFY_LEFT, GR_TEXT_VJUSTIFY_TOP, LineWidth,
+                     false, false, false );
 
 
     /* Draw text : SheetLabel */
@@ -352,13 +337,10 @@ void DrawSheetStruct::Draw( WinEDA_DrawPanel* aPanel, wxDC* aDC,
 }
 
 
-/*****************************************/
-EDA_Rect DrawSheetStruct::GetBoundingBox()
-/*****************************************/
-
 /** Function GetBoundingBox
- *  @return an EDA_Rect giving the bouding box of the sheet
+ *  @return an EDA_Rect giving the bounding box of the sheet
  */
+EDA_Rect SCH_SHEET::GetBoundingBox()
 {
     int      dx, dy;
 
@@ -372,31 +354,29 @@ EDA_Rect DrawSheetStruct::GetBoundingBox()
     dx = MAX( m_Size.x, textlen1 );
     dy = m_Size.y + m_SheetNameSize + m_FileNameSize + 16;
 
-    EDA_Rect box( wxPoint( m_Pos.x, m_Pos.y - m_SheetNameSize - 8 ), wxSize( dx, dy ) );
+    EDA_Rect box( wxPoint( m_Pos.x, m_Pos.y - m_SheetNameSize - 8 ),
+                  wxSize( dx, dy ) );
     return box;
 }
 
-/************************************************/
-bool DrawSheetStruct::HitTest( const wxPoint& aPosRef )
-/************************************************/
+
 /** Function HitTest
  * @return true if the point aPosRef is within item area
  * @param aPosRef = a wxPoint to test
  */
+bool SCH_SHEET::HitTest( const wxPoint& aPosRef )
 {
     EDA_Rect rect = GetBoundingBox();
+
     return rect.Inside( aPosRef );
 }
 
 
-/************************************/
-int DrawSheetStruct::ComponentCount()
-/************************************/
-
 /** Function ComponentCount
  *  count our own components, without the power components.
- *  @return the copponent count.
+ *  @return the component count.
  */
+int SCH_SHEET::ComponentCount()
 {
     int n = 0;
 
@@ -413,25 +393,23 @@ int DrawSheetStruct::ComponentCount()
             }
             if( bs->Type() == DRAW_SHEET_STRUCT_TYPE )
             {
-                DrawSheetStruct* sheet = (DrawSheetStruct*) bs;
+                SCH_SHEET* sheet = (SCH_SHEET*) bs;
                 n += sheet->ComponentCount();
             }
         }
     }
+
     return n;
 }
 
 
-/*******************************************************************************/
-bool DrawSheetStruct::SearchHierarchy( wxString aFilename, SCH_SCREEN** aScreen )
-/*******************************************************************************/
-
 /** Function SearchHierarchy
  *  search the existing hierarchy for an instance of screen "FileName".
  *  @param aFilename = the filename to find
  *  @param aFilename = a location to return a pointer to the screen (if found)
  *  @return bool if found, and a pointer to the screen
  */
+bool SCH_SHEET::SearchHierarchy( wxString aFilename, SCH_SCREEN** aScreen )
 {
     if( m_AssociatedScreen )
     {
@@ -440,37 +418,36 @@ bool DrawSheetStruct::SearchHierarchy( wxString aFilename, SCH_SCREEN** aScreen
         {
             if( strct->Type() == DRAW_SHEET_STRUCT_TYPE )
             {
-                DrawSheetStruct* ss = (DrawSheetStruct*) strct;
+                SCH_SHEET* ss = (SCH_SHEET*) strct;
                 if( ss->m_AssociatedScreen
                     && ss->m_AssociatedScreen->m_FileName.CmpNoCase( aFilename ) == 0 )
                 {
                     *aScreen = ss->m_AssociatedScreen;
                     return true;
                 }
+
                 if( ss->SearchHierarchy( aFilename, aScreen ) )
                     return true;
             }
             strct = strct->Next();
         }
     }
+
     return false;
 }
 
 
-/*******************************************************************************/
-bool DrawSheetStruct::LocatePathOfScreen( SCH_SCREEN*    aScreen,
-                                          DrawSheetPath* aList )
-/*******************************************************************************/
-
 /** Function LocatePathOfScreen
  *  search the existing hierarchy for an instance of screen "FileName".
  *  don't bother looking at the root sheet - it must be unique,
- *  no other references to its m_AssociatedScreen otherwise there would be loops
+ *  no other references to its m_AssociatedScreen otherwise there would be
+ * loops
  *  in the hierarchy.
  *  @param  aScreen = the SCH_SCREEN* screen that we search for
- *  @param aList = the DrawSheetPath*  that must be used
+ *  @param aList = the SCH_SHEET_PATH*  that must be used
  *  @return true if found
  */
+bool SCH_SHEET::LocatePathOfScreen( SCH_SCREEN* aScreen, SCH_SHEET_PATH* aList )
 {
     if( m_AssociatedScreen )
     {
@@ -482,7 +459,7 @@ bool DrawSheetStruct::LocatePathOfScreen( SCH_SCREEN*    aScreen,
         {
             if( strct->Type() == DRAW_SHEET_STRUCT_TYPE )
             {
-                DrawSheetStruct* ss = (DrawSheetStruct*) strct;
+                SCH_SHEET* ss = (SCH_SHEET*) strct;
                 if( ss->LocatePathOfScreen( aScreen, aList ) )
                     return true;
             }
@@ -495,10 +472,6 @@ bool DrawSheetStruct::LocatePathOfScreen( SCH_SCREEN*    aScreen,
 }
 
 
-/**********************************************************/
-bool DrawSheetStruct::Load( WinEDA_SchematicFrame* aFrame )
-/***********************************************************/
-
 /** Function Load.
  *  for the sheet: load the file m_FileName
  *  if a screen already exists, the file is already read.
@@ -507,6 +480,7 @@ bool DrawSheetStruct::Load( WinEDA_SchematicFrame* aFrame )
  *  @param aFrame = a WinEDA_SchematicFrame pointer to the maim schematic frame
  *  @return true if OK
  */
+bool SCH_SHEET::Load( WinEDA_SchematicFrame* aFrame )
 {
     bool success = true;
 
@@ -533,7 +507,7 @@ bool DrawSheetStruct::Load( WinEDA_SchematicFrame* aFrame )
                 {
                     if( bs->Type() ==  DRAW_SHEET_STRUCT_TYPE )
                     {
-                        DrawSheetStruct* sheetstruct = (DrawSheetStruct*) bs;
+                        SCH_SHEET* sheetstruct = (SCH_SHEET*) bs;
                         if( !sheetstruct->Load( aFrame ) )
                             success = false;
                     }
@@ -542,18 +516,17 @@ bool DrawSheetStruct::Load( WinEDA_SchematicFrame* aFrame )
             }
         }
     }
+
     return success;
 }
 
 
-/**********************************/
-int DrawSheetStruct::CountSheets()
-/**********************************/
 /** Function CountSheets
  * calculates the number of sheets found in "this"
  * this number includes the full subsheets count
  * @return the full count of sheets+subsheets contained by "this"
  */
+int SCH_SHEET::CountSheets()
 {
     int count = 1; //1 = this!!
 
@@ -564,7 +537,7 @@ int DrawSheetStruct::CountSheets()
         {
             if( strct->Type() == DRAW_SHEET_STRUCT_TYPE )
             {
-                DrawSheetStruct* subsheet = (DrawSheetStruct*) strct;
+                SCH_SHEET* subsheet = (SCH_SHEET*) strct;
                 count += subsheet->CountSheets();
             }
         }
@@ -573,28 +546,26 @@ int DrawSheetStruct::CountSheets()
 }
 
 
-/******************************************/
-wxString DrawSheetStruct::GetFileName( void )
-/******************************************/
+wxString SCH_SHEET::GetFileName( void )
 {
     return m_FileName;
 }
 
 
-/************************************************************************/
-bool DrawSheetStruct::ChangeFileName( WinEDA_SchematicFrame* aFrame,
-                                      const wxString&        aFileName )
-/************************************************************************/
 /** Function ChangeFileName
  * Set a new filename and manage data and associated screen
  * The main difficulty is the filename change in a complex hierarchy.
- * - if new filename is not already used: change to the new name (and if an existing file is found, load it on request)
- * - if new filename is already used (a complex hierarchy) : reference the sheet.
+ * - if new filename is not already used: change to the new name (and if an
+ * existing file is found, load it on request)
+ * - if new filename is already used (a complex hierarchy) : reference the
+ * sheet.
  * @param aFileName = the new filename
  * @param aFrame = the schematic frame
  */
+bool SCH_SHEET::ChangeFileName( WinEDA_SchematicFrame* aFrame,
+                                const wxString&        aFileName )
 {
-    if( (GetFileName() == aFileName) && m_AssociatedScreen )
+    if( ( GetFileName() == aFileName ) && m_AssociatedScreen )
         return true;
 
     SCH_SCREEN* Screen_to_use = NULL;
@@ -602,45 +573,52 @@ bool DrawSheetStruct::ChangeFileName( WinEDA_SchematicFrame* aFrame,
     bool        LoadFromFile = false;
 
 
-    if( g_RootSheet->SearchHierarchy( aFileName, &Screen_to_use ) ) //do we reload the data from the existing hierarchy
+    // do we reload the data from the existing hierarchy
+    if( g_RootSheet->SearchHierarchy( aFileName, &Screen_to_use ) )
     {
-        if( m_AssociatedScreen )                                    //upon initial load, this will be null.
+        if( m_AssociatedScreen ) // upon initial load, this will be null.
         {
-            msg.Printf( _(
-                           "A Sub Hierarchy named %s exists, Use it (The data in this sheet will be replaced)?" ),
-                       aFileName.GetData() );
+            msg.Printf( _( "A Sub Hierarchy named %s exists, Use it (The \
+data in this sheet will be replaced)?" ),
+                        aFileName.GetData() );
             if( !IsOK( NULL, msg ) )
             {
-                DisplayInfo( NULL, _( "Sheet Filename Renaming Aborted" ) );
+                DisplayInfoMessage( (wxWindow*) NULL,
+                                    _( "Sheet Filename Renaming Aborted" ) );
                 return false;
             }
         }
     }
-    else if( wxFileExists( aFileName ) )         //do we reload the data from an existing file
+    else if( wxFileExists( aFileName ) )         // do we reload the data from
+                                                 // an existing file
     {
-        msg.Printf( _(
-                       "A file named %s exists, load it (otherwise keep current sheet data if possible)?" ),
+        msg.Printf( _( "A file named %s exists, load it (otherwise keep \
+current sheet data if possible)?" ),
                    aFileName.GetData() );
         if( IsOK( NULL, msg ) )
         {
             LoadFromFile = true;
-            if( m_AssociatedScreen )                                // Can be NULL if loading a file when creating a new sheet
+
+            // Can be NULL if loading a file when creating a new sheet.
+            if( m_AssociatedScreen )
             {
-                m_AssociatedScreen->m_RefCount--;                   //be careful with these
+                m_AssociatedScreen->m_RefCount--;  // be careful with these
                 if( m_AssociatedScreen->m_RefCount == 0 )
                     SAFE_DELETE( m_AssociatedScreen );
-                m_AssociatedScreen = NULL;         //will be created later
+                m_AssociatedScreen = NULL;         // will be created later
             }
         }
     }
 
-    // if an associated screen exists, shared between this sheet and others sheets, what we do ?
+    // if an associated screen exists, shared between this sheet and others
+    // sheets, what we do ?
     if( m_AssociatedScreen && ( m_AssociatedScreen->m_RefCount > 1 ) )
     {
         msg = _( "This sheet uses shared data in a complex hierarchy" );
         msg << wxT( "\n" );
-        msg << _(
-            "Do we convert it in a simple hierarchical sheet (otherwise delete current sheet data)" );
+        msg << _( "Do we convert it in a simple hierarchical sheet (\
+otherwise delete current sheet data)" );
+
         if( IsOK( NULL, msg ) )
         {
             LoadFromFile = true;
@@ -656,13 +634,14 @@ bool DrawSheetStruct::ChangeFileName( WinEDA_SchematicFrame* aFrame,
 
     SetFileName( aFileName );
 
-    // if we use new data (from file or from internal hierarchy), delete the current sheet data
+    // if we use new data (from file or from internal hierarchy), delete the
+    // current sheet data
     if( m_AssociatedScreen && (LoadFromFile || Screen_to_use) )
     {
         m_AssociatedScreen->m_RefCount--;
         if( m_AssociatedScreen->m_RefCount == 0 )
             SAFE_DELETE( m_AssociatedScreen );
-        m_AssociatedScreen = NULL;         //so that we reload..
+        m_AssociatedScreen = NULL;         // so that we reload..
     }
 
     if( LoadFromFile )
@@ -678,7 +657,7 @@ bool DrawSheetStruct::ChangeFileName( WinEDA_SchematicFrame* aFrame,
     if( !m_AssociatedScreen )
     {
         m_AssociatedScreen = new SCH_SCREEN();
-        m_AssociatedScreen->m_RefCount++;         //be careful with these
+        m_AssociatedScreen->m_RefCount++;         // be careful with these
     }
     m_AssociatedScreen->m_FileName = aFileName;
 
@@ -686,19 +665,49 @@ bool DrawSheetStruct::ChangeFileName( WinEDA_SchematicFrame* aFrame,
 }
 
 
+void SCH_SHEET::DisplayInfo( WinEDA_DrawFrame* frame )
+{
+    frame->ClearMsgPanel();
+    frame->AppendMsgPanel( _( "Sheet name" ), m_SheetName, CYAN );
+    frame->AppendMsgPanel( _( "File name" ), m_FileName, BROWN );
+}
+
+
+/** virtual function Mirror_Y
+ * mirror item relative to an Y axis
+ * @param aYaxis_position = the y axis position
+ */
+void SCH_SHEET::Mirror_Y( int aYaxis_position )
+{
+    m_Pos.x -= aYaxis_position;
+    NEGATE( m_Pos.x );
+    m_Pos.x += aYaxis_position;
+
+    m_Pos.x -= m_Size.x;
+
+    SCH_SHEET_PIN* label = m_Label;
+    while( label != NULL )
+    {
+        label->Mirror_Y( aYaxis_position );
+        label = label->Next();
+    }
+}
+
+
 #if defined(DEBUG)
-void DrawSheetStruct::Show( int nestLevel, std::ostream& os )
+void SCH_SHEET::Show( int nestLevel, std::ostream& os )
 {
     // XML output:
     wxString s = GetClass();
 
     NestedSpace( nestLevel, os ) << '<' << s.Lower().mb_str() << ">"
-                                 << " sheet_name=\"" << CONV_TO_UTF8( m_SheetName ) << '"'
-                                 << ">\n";
+                                 << " sheet_name=\""
+                                 << CONV_TO_UTF8( m_SheetName )
+                                 << '"' << ">\n";
 
     // show all the pins, and check the linked list integrity
-    Hierarchical_PIN_Sheet_Struct* label;
-    for( label = m_Label;   label;   label = label->Next() )
+    SCH_SHEET_PIN* label;
+    for( label = m_Label; label; label = label->Next() )
     {
         label->Show( nestLevel + 1, os );
     }
