@@ -1,5 +1,5 @@
 /****************************************************************/
-/* EESchema: find.cpp (functions for seraching a schematic item */
+/* EESchema: find.cpp (functions for searching a schematic item */
 /****************************************************************/
 
 /*
@@ -8,15 +8,44 @@
  *  in current sheet or whole the project
  */
 #include "fctsys.h"
-#include "gr_basic.h"
+#include "appl_wxstruct.h"
 #include "common.h"
 #include "class_drawpanel.h"
 #include "confirm.h"
 #include "kicad_string.h"
 #include "gestfich.h"
+
 #include "program.h"
-#include "libcmp.h"
 #include "general.h"
+#include "class_marker_sch.h"
+#include "protos.h"
+#include "class_library.h"
+#include "class_pin.h"
+
+#include "kicad_device_context.h"
+
+#include <wx/fdrepdlg.h>          // Use the wxFindReplaceDialog events, data, and enums.
+#include <boost/foreach.hpp>
+
+
+/* Define schematic specific find and replace dialog flags based on the enum entries
+ * in wxFindReplaceFlags.   These flags are intended to be used as bit masks in the
+ * wxFindReplaceData::m_Flags member variable.  The varialble is defined as a wxUint32.
+ */
+enum SchematicFindReplaceFlags
+{
+    /* The last wxFindReplaceFlag enum is wxFR_MATCHCASE. */
+
+    /* Search the current sheet only. */
+    schFR_CURRENT_SHEET_ONLY        = wxFR_MATCHCASE << 1,
+
+    /* Search for design rule check markers. */
+    schFR_DRC_MARKERS               = wxFR_MATCHCASE << 2,
+
+    /* Search for component in all loaded libraries. */
+    schFR_SEARCH_LIBS_FOR_COMPONENT = wxFR_MATCHCASE << 3
+};
+
 
 /* Variables Locales */
 static int      s_ItemsCount, s_MarkerCount;
@@ -25,17 +54,11 @@ static wxString s_OldStringFound;
 #include "dialog_find.cpp"
 
 
-#include "protos.h"
-
-
-/**************************************************************/
-void WinEDA_FindFrame::FindMarker( wxCommandEvent& event )
-/**************************************************************/
-
 /*  Search markers in whole hierarchy.
  *  Mouse cursor is put on the marker
  *  search the first marker, or next marker
  */
+void WinEDA_FindFrame::FindMarker( wxCommandEvent& event )
 {
     int id = event.GetId();
 
@@ -48,19 +71,12 @@ void WinEDA_FindFrame::FindMarker( wxCommandEvent& event )
 }
 
 
-/************************************************************************/
-SCH_ITEM * WinEDA_SchematicFrame::FindComponentAndItem(
-    const wxString& component_reference, bool Find_in_hierarchy,
-    int SearchType,
-    const wxString& text_to_find,
-    bool mouseWarp )
-/************************************************************************/
-
 /**
  * Function FindComponentAndItem
  * finds a Component in the schematic, and an item in this component.
  * @param component_reference The component reference to find.
- * @param text_to_find The text to search for, either in value, reference or elsewhere.
+ * @param text_to_find - The text to search for, either in value, reference
+ *                       or elsewhere.
  * @param Find_in_hierarchy:  false => Search is made in current sheet
  *                     true => the whole hierarchy
  * @param SearchType:  0 => find component
@@ -70,18 +86,22 @@ SCH_ITEM * WinEDA_SchematicFrame::FindComponentAndItem(
  *                     >= 4 => unused (same as 0)
  * @param mouseWarp If true, then move the mouse cursor to the item.
  */
+SCH_ITEM* WinEDA_SchematicFrame::FindComponentAndItem(
+    const wxString& component_reference, bool Find_in_hierarchy,
+    int SearchType,
+    const wxString& text_to_find,
+    bool mouseWarp )
 {
-    DrawSheetPath*          sheet, * SheetWithComponentFound = NULL;
-    SCH_ITEM*               DrawList  = NULL;
-    SCH_COMPONENT* Component = NULL;
-    wxSize                  DrawAreaSize = DrawPanel->GetClientSize();
-    wxPoint                 pos, curpos;
-    bool                    DoCenterAndRedraw = FALSE;
-    bool                    NotFound = true;
-    wxString                msg;
-    LibDrawPin*             pin;
-
-    EDA_SheetList          SheetList;
+    SCH_SHEET_PATH* sheet, * SheetWithComponentFound = NULL;
+    SCH_ITEM*       DrawList     = NULL;
+    SCH_COMPONENT*  Component    = NULL;
+    wxSize          DrawAreaSize = DrawPanel->GetClientSize();
+    wxPoint         pos, curpos;
+    bool            DoCenterAndRedraw = FALSE;
+    bool            NotFound = true;
+    wxString        msg;
+    LIB_PIN*        pin;
+    SCH_SHEET_LIST  SheetList;
 
     sheet = SheetList.GetFirst();
     if( !Find_in_hierarchy )
@@ -90,13 +110,14 @@ SCH_ITEM * WinEDA_SchematicFrame::FindComponentAndItem(
     for( ; sheet != NULL; sheet = SheetList.GetNext() )
     {
         DrawList = (SCH_ITEM*) sheet->LastDrawList();
-        for( ; (DrawList != NULL) && (NotFound == true); DrawList = DrawList->Next() )
+        for( ; ( DrawList != NULL ) && ( NotFound == true );
+             DrawList = DrawList->Next() )
         {
             if( DrawList->Type() == TYPE_SCH_COMPONENT )
             {
                 SCH_COMPONENT* pSch;
                 pSch = (SCH_COMPONENT*) DrawList;
-                if( component_reference.CmpNoCase( pSch->GetRef(sheet) ) == 0 )
+                if( component_reference.CmpNoCase( pSch->GetRef( sheet ) ) == 0 )
                 {
                     Component = pSch;
                     SheetWithComponentFound = sheet;
@@ -110,8 +131,9 @@ SCH_ITEM * WinEDA_SchematicFrame::FindComponentAndItem(
                         break;
 
                     case 1:                 // find a pin
-                        pos = pSch->m_Pos;  // temporary: will be changed if the pin is found
-                        pin = LocatePinByNumber( text_to_find, pSch );
+                        pos = pSch->m_Pos;  /* temporary: will be changed if
+                                             * the pin is found */
+                        pin = pSch->GetPin( text_to_find );
                         if( pin == NULL )
                             break;
                         NotFound = FALSE;
@@ -146,43 +168,41 @@ SCH_ITEM * WinEDA_SchematicFrame::FindComponentAndItem(
         {
             sheet->LastScreen()->SetZoom( GetScreen()->GetZoom() );
             *m_CurrentSheet = *sheet;
-            ActiveScreen = m_CurrentSheet->LastScreen();
+            ActiveScreen    = m_CurrentSheet->LastScreen();
             m_CurrentSheet->UpdateAllScreenReferences();
-            DoCenterAndRedraw   = TRUE;
+            DoCenterAndRedraw = TRUE;
         }
         wxPoint delta;
-        pos -= Component->m_Pos;
-        delta = TransformCoordinate( Component->m_Transform, pos);
-        pos = delta + Component->m_Pos;
+        pos  -= Component->m_Pos;
+        delta = TransformCoordinate( Component->m_Transform, pos );
+        pos   = delta + Component->m_Pos;
 
-        wxPoint old_cursor_position = sheet->LastScreen()->m_Curseur;
+        wxPoint old_cursor_position    = sheet->LastScreen()->m_Curseur;
         sheet->LastScreen()->m_Curseur = pos;
 
         curpos = DrawPanel->CursorScreenPosition();
 
         DrawPanel->GetViewStart(
             &( GetScreen()->m_StartVisu.x ),
-            &( GetScreen()->m_StartVisu.y ));
+            &( GetScreen()->m_StartVisu.y ) );
 
-        // calcul des coord curseur avec origine = screen
+        // Calculating cursor position with original screen.
         curpos -= GetScreen()->m_StartVisu;
 
-        /* Il y a peut-etre necessite de recadrer le dessin: */
+        /* There may be need to reframe the drawing */
         #define MARGIN 30
-        if( (curpos.x <= MARGIN) || (curpos.x >= DrawAreaSize.x - MARGIN)
-           || (curpos.y <= MARGIN) || (curpos.y >= DrawAreaSize.y - MARGIN) )
+        if( ( curpos.x <= MARGIN ) || ( curpos.x >= DrawAreaSize.x - MARGIN )
+           || ( curpos.y <= MARGIN ) || ( curpos.y >= DrawAreaSize.y - MARGIN ) )
         {
             DoCenterAndRedraw = true;;
         }
         #undef MARGIN
 
-        if ( DoCenterAndRedraw )
+        if( DoCenterAndRedraw )
             Recadre_Trace( mouseWarp );
         else
         {
-            wxClientDC  dc( DrawPanel );
-
-            DrawPanel->PrepareGraphicContext( &dc );
+            INSTALL_DC( dc, DrawPanel );
 
             EXCHG( old_cursor_position, sheet->LastScreen()->m_Curseur );
             DrawPanel->CursorOff( &dc );
@@ -254,57 +274,56 @@ SCH_ITEM * WinEDA_SchematicFrame::FindComponentAndItem(
 }
 
 
-/*****************************************************************/
-SCH_ITEM * WinEDA_SchematicFrame::FindMarker( int SearchType )
-/*****************************************************************/
-
 /* Search markers in whole the hierarchy.
  *  Mouse cursor is put on the marker
  *  SearchType = 0: search the first marker, else search next marker
  */
+SCH_ITEM* WinEDA_SchematicFrame::FindMarker( int SearchType )
 {
-    DrawSheetPath*  sheet, * FirstSheet = NULL;
-    SCH_ITEM*   DrawList, * FirstStruct = NULL, * Struct = NULL;
-    DrawMarkerStruct * Marker = NULL;
-    int StartCount;
-    bool NotFound;
-    wxPoint           firstpos, pos;
-    wxSize            DrawAreaSize = DrawPanel->GetClientSize();
-    wxPoint           curpos, old_cursor_position;
-    bool DoCenterAndRedraw = FALSE;
-    wxString          msg, WildText;
+    SCH_SHEET_PATH* sheet, * FirstSheet = NULL;
+    SCH_ITEM*       DrawList, * FirstStruct = NULL, * Struct = NULL;
+    SCH_MARKER*     Marker = NULL;
+    int             StartCount;
+    bool            NotFound;
+    wxPoint         firstpos, pos;
+    wxSize          DrawAreaSize = DrawPanel->GetClientSize();
+    wxPoint         curpos, old_cursor_position;
+    bool            DoCenterAndRedraw = FALSE;
+    wxString        msg, WildText;
 
     g_LastSearchIsMarker = TRUE;
     /* Set s_MarkerCount to 0 if we are look for the first marker */
     if( SearchType == 0 )
         s_MarkerCount = 0;
 
-    EDA_SheetList SheetList;
+    SCH_SHEET_LIST SheetList;
 
     NotFound = TRUE; StartCount = 0;
     /* Search for s_MarkerCount markers */
-    for( sheet = SheetList.GetFirst(); sheet != NULL; sheet = SheetList.GetNext() )
+    for( sheet = SheetList.GetFirst(); sheet != NULL;
+         sheet = SheetList.GetNext() )
     {
         DrawList = (SCH_ITEM*) sheet->LastDrawList();
         while( DrawList && NotFound )
         {
-            if( DrawList->Type() == DRAW_MARKER_STRUCT_TYPE )
+            if( DrawList->Type() == TYPE_SCH_MARKER )
             {
-                Marker   = (DrawMarkerStruct*) DrawList;
+                Marker   = (SCH_MARKER*) DrawList;
                 NotFound = FALSE;
                 pos = Marker->m_Pos;
                 if( FirstSheet == NULL )    /* First item found */
                 {
-                    FirstSheet = sheet; firstpos = pos;
+                    FirstSheet  = sheet; firstpos = pos;
                     FirstStruct = DrawList;
                 }
 
                 StartCount++;
                 if( s_MarkerCount >= StartCount )
                 {
-                    NotFound = TRUE;        /* Search for other markers */
+                    NotFound = TRUE;       /* Search for other markers */
                 }
-                else                        /* We have found s_MarkerCount markers -> Ok */
+                else                        /* We have found s_MarkerCount
+                                             * markers -> Ok */
                 {
                     Struct = DrawList; s_MarkerCount++; break;
                 }
@@ -316,8 +335,10 @@ SCH_ITEM * WinEDA_SchematicFrame::FindMarker( int SearchType )
             break;
     }
 
-    if( NotFound && FirstSheet )       // markers are found, but we have reach the last marker */
-    {                                   // After the last marker, the first marker is used */
+    if( NotFound && FirstSheet )       /* markers are found, but we have
+                                        * reach the last marker */
+    {
+        // After the last marker, the first marker is used */
         NotFound = FALSE; sheet = FirstSheet;
         Struct   = FirstStruct;
         pos = firstpos; s_MarkerCount = 1;
@@ -329,25 +350,24 @@ SCH_ITEM * WinEDA_SchematicFrame::FindMarker( int SearchType )
         {
             sheet->LastScreen()->SetZoom( GetScreen()->GetZoom() );
             *m_CurrentSheet = *sheet;
-            ActiveScreen = m_CurrentSheet->LastScreen();
+            ActiveScreen    = m_CurrentSheet->LastScreen();
             m_CurrentSheet->UpdateAllScreenReferences();
-            DoCenterAndRedraw   = TRUE;
+            DoCenterAndRedraw = TRUE;
         }
 
         old_cursor_position = sheet->LastScreen()->m_Curseur;
-        sheet->LastScreen()->m_Curseur   = pos;
+        sheet->LastScreen()->m_Curseur = pos;
         curpos = DrawPanel->CursorScreenPosition();
 
-        // calcul des coord curseur avec origine = screen
         DrawPanel->GetViewStart( &m_CurrentSheet->LastScreen()->m_StartVisu.x,
-                                  &m_CurrentSheet->LastScreen()->m_StartVisu.y );
+                                 &m_CurrentSheet->LastScreen()->m_StartVisu.y );
         curpos.x -= m_CurrentSheet->LastScreen()->m_StartVisu.x;
         curpos.y -= m_CurrentSheet->LastScreen()->m_StartVisu.y;
 
         // reposition the window if the chosen marker is off screen.
         #define MARGIN 30
-        if( (curpos.x <= MARGIN) || (curpos.x >= DrawAreaSize.x - MARGIN)
-           || (curpos.y <= MARGIN) || (curpos.y >= DrawAreaSize.y - MARGIN) )
+        if( ( curpos.x <= MARGIN ) || ( curpos.x >= DrawAreaSize.x - MARGIN )
+           || ( curpos.y <= MARGIN ) || ( curpos.y >= DrawAreaSize.y - MARGIN ) )
         {
             DoCenterAndRedraw = true;;
         }
@@ -357,9 +377,7 @@ SCH_ITEM * WinEDA_SchematicFrame::FindMarker( int SearchType )
             Recadre_Trace( TRUE );
         else
         {
-            wxClientDC dc( DrawPanel );
-
-            DrawPanel->PrepareGraphicContext( &dc );
+            INSTALL_DC( dc, DrawPanel );
             EXCHG( old_cursor_position, sheet->LastScreen()->m_Curseur );
             DrawPanel->CursorOff( &dc );
             DrawPanel->MouseTo( curpos );
@@ -367,7 +385,8 @@ SCH_ITEM * WinEDA_SchematicFrame::FindMarker( int SearchType )
             DrawPanel->CursorOn( &dc );
         }
         wxString path = sheet->Path();
-        msg.Printf( _( "Marker %d found in %s" ), s_MarkerCount, path.GetData() );
+        msg.Printf( _( "Marker %d found in %s" ),
+                    s_MarkerCount, path.GetData() );
         Affiche_Message( msg );
     }
     else
@@ -381,13 +400,10 @@ SCH_ITEM * WinEDA_SchematicFrame::FindMarker( int SearchType )
 }
 
 
-/**************************************************************/
-void WinEDA_FindFrame::FindSchematicItem( wxCommandEvent& event )
-/**************************************************************/
-
 /* Find a string in schematic.
  *  Call to WinEDA_SchematicFrame::FindSchematicItem()
  */
+void WinEDA_FindFrame::FindSchematicItem( wxCommandEvent& event )
 {
     int id = event.GetId();
 
@@ -402,23 +418,22 @@ void WinEDA_FindFrame::FindSchematicItem( wxCommandEvent& event )
 }
 
 
-/************************************************************************/
-SCH_ITEM* WinEDA_SchematicFrame::FindSchematicItem(
-    const wxString& pattern, int SearchType, bool mouseWarp )
-/************************************************************************/
-
 /**
  * Function FindSchematicItem
  * finds a string in the schematic.
- * @param pattern The text to search for, either in value, reference or elsewhere.
+ * @param pattern The text to search for, either in value, reference or
+ *                 elsewhere.
  * @param SearchType:  0 => Search is made in current sheet
  *                     1 => the whole hierarchy
  *                     2 => or for the next item
  * @param mouseWarp If true, then move the mouse cursor to the item.
  */
+SCH_ITEM* WinEDA_SchematicFrame::FindSchematicItem( const wxString& pattern,
+                                                    int             SearchType,
+                                                    bool            mouseWarp )
 {
-    DrawSheetPath*     Sheet, * FirstSheet = NULL;
-    SCH_ITEM* DrawList = NULL, * FirstStruct = NULL, * Struct = NULL;
+    SCH_SHEET_PATH* Sheet, * FirstSheet = NULL;
+    SCH_ITEM*       DrawList = NULL, * FirstStruct = NULL, * Struct = NULL;
     int             StartCount;
     bool            NotFound;
     wxPoint         firstpos, pos, old_cursor_position;
@@ -449,7 +464,7 @@ SCH_ITEM* WinEDA_SchematicFrame::FindSchematicItem(
     NotFound   = TRUE;
     StartCount = 0;
 
-    EDA_SheetList SheetList;
+    SCH_SHEET_LIST SheetList;
 
     Sheet = SheetList.GetFirst();
     if( !Find_in_hierarchy )
@@ -457,7 +472,7 @@ SCH_ITEM* WinEDA_SchematicFrame::FindSchematicItem(
 
     for( ; Sheet != NULL; Sheet = SheetList.GetNext() )
     {
-        DrawList = (SCH_ITEM*)Sheet->LastDrawList();
+        DrawList = (SCH_ITEM*) Sheet->LastDrawList();
         while( DrawList )
         {
             switch( DrawList->Type() )
@@ -465,13 +480,15 @@ SCH_ITEM* WinEDA_SchematicFrame::FindSchematicItem(
             case TYPE_SCH_COMPONENT:
                 SCH_COMPONENT * pSch;
                 pSch = (SCH_COMPONENT*) DrawList;
-                if( WildCompareString( WildText, pSch->GetRef(Sheet), FALSE ) )
+                if( WildCompareString( WildText, pSch->GetRef( Sheet ), FALSE ) )
                 {
                     NotFound = FALSE;
                     pos = pSch->GetField( REFERENCE )->m_Pos;
                     break;
                 }
-                if( WildCompareString( WildText, pSch->GetField( VALUE )->m_Text, FALSE ) )
+                if( WildCompareString( WildText,
+                                       pSch->GetField( VALUE )->m_Text,
+                                       false ) )
                 {
                     NotFound = FALSE;
                     pos = pSch->GetField( VALUE )->m_Pos;
@@ -497,9 +514,9 @@ SCH_ITEM* WinEDA_SchematicFrame::FindSchematicItem(
 
             if( NotFound == FALSE )             /* Item found ! */
             {
-                if( FirstSheet == NULL )       /* First Item found */
+                if( FirstSheet == NULL )        /* First Item found */
                 {
-                    FirstSheet = Sheet;
+                    FirstSheet  = Sheet;
                     firstpos    = pos;
                     FirstStruct = DrawList;
                 }
@@ -507,7 +524,7 @@ SCH_ITEM* WinEDA_SchematicFrame::FindSchematicItem(
                 StartCount++;
                 if( s_ItemsCount >= StartCount )
                 {
-                    NotFound = TRUE;    /* Continue recherche de l'element suivant */
+                    NotFound = TRUE;    /* Continue search of the next element */
                 }
                 else
                 {
@@ -531,7 +548,7 @@ SCH_ITEM* WinEDA_SchematicFrame::FindSchematicItem(
     if( NotFound && FirstSheet )
     {
         NotFound = FALSE;
-        Sheet   = FirstSheet;
+        Sheet    = FirstSheet;
         Struct   = FirstStruct;
         pos = firstpos;
         s_ItemsCount = 1;
@@ -543,9 +560,9 @@ SCH_ITEM* WinEDA_SchematicFrame::FindSchematicItem(
         {
             Sheet->LastScreen()->SetZoom( GetScreen()->GetZoom() );
             *m_CurrentSheet = *Sheet;
-            ActiveScreen = m_CurrentSheet->LastScreen();
+            ActiveScreen    = m_CurrentSheet->LastScreen();
             m_CurrentSheet->UpdateAllScreenReferences();
-            DoCenterAndRedraw   = TRUE;
+            DoCenterAndRedraw = TRUE;
         }
 
         /* the struct is a TYPE_SCH_COMPONENT type,
@@ -556,37 +573,34 @@ SCH_ITEM* WinEDA_SchematicFrame::FindSchematicItem(
             SCH_COMPONENT* pSch = (SCH_COMPONENT*) Struct;
 
             pos -= pSch->m_Pos;
-            pos = TransformCoordinate( pSch->m_Transform, pos );
+            pos  = TransformCoordinate( pSch->m_Transform, pos );
             pos += pSch->m_Pos;
         }
 
         old_cursor_position = Sheet->LastScreen()->m_Curseur;
-        Sheet->LastScreen()->m_Curseur   = pos;
+        Sheet->LastScreen()->m_Curseur = pos;
 
         curpos = DrawPanel->CursorScreenPosition();
 
         DrawPanel->GetViewStart(
-                &( GetScreen()->m_StartVisu.x ),
-                &( GetScreen()->m_StartVisu.y ));
+            &( GetScreen()->m_StartVisu.x ),
+            &( GetScreen()->m_StartVisu.y ) );
 
-        // calcul des coord curseur avec origine = screen
         curpos -= m_CurrentSheet->LastScreen()->m_StartVisu;
 
-        /* Il y a peut-etre necessite de recadrer le dessin: */
+        /* There may be need to reframe the drawing */
         #define MARGIN 30
         if( (curpos.x <= MARGIN) || (curpos.x >= DrawAreaSize.x - MARGIN)
            || (curpos.y <= MARGIN) || (curpos.y >= DrawAreaSize.y - MARGIN) )
         {
-           DoCenterAndRedraw = true;
+            DoCenterAndRedraw = true;
         }
 
-        if ( DoCenterAndRedraw )
+        if( DoCenterAndRedraw )
             Recadre_Trace( mouseWarp );
         else
         {
-            wxClientDC  dc( DrawPanel );
-
-            DrawPanel->PrepareGraphicContext( &dc );
+            INSTALL_DC( dc, DrawPanel );
 
             EXCHG( old_cursor_position, Sheet->LastScreen()->m_Curseur );
             DrawPanel->CursorOff( &dc );
@@ -609,7 +623,7 @@ SCH_ITEM* WinEDA_SchematicFrame::FindSchematicItem(
         if( !mouseWarp )
         {
             // if called from RemoteCommand() don't popup the dialog which
-            // needs to be dismissed, user is in PCBNEW, and does'nt want to
+            // needs to be dismissed, user is in PCBNEW, and doesn't want to
             // bother with dismissing the dialog in EESCHEMA.
             msg = WildText + _( " Not Found" );
             DisplayError( this, msg, 10 );
@@ -620,69 +634,61 @@ SCH_ITEM* WinEDA_SchematicFrame::FindSchematicItem(
 }
 
 
-/*************************************************************/
-void WinEDA_FindFrame::LocatePartInLibs( wxCommandEvent& event )
-/*************************************************************/
-
-/* Recherche exhaustive d'un composant en librairies, meme non chargees
+/*
+ * Search for a given component.
+ *
+ * The search is made in loaded libraries, and if not found in all libraries
+ * found in lib paths.
  */
+void WinEDA_FindFrame::LocatePartInLibs( wxCommandEvent& event )
 {
-    wxString                Text, FindList;
-    const wxChar**          ListNames;
-    LibraryStruct*          Lib = NULL;
-    EDA_LibComponentStruct* LibEntry;
-    bool FoundInLib = FALSE;     // True si reference trouvee ailleurs qu'en cache
+    wxArrayString nameList;
+    wxString      Text, FindList;
+    bool          FoundInLib = false;
 
     Text = m_NewTextCtrl->GetValue();
     if( Text.IsEmpty() )
     {
-        Close(); return;
+        Close();
+        return;
     }
+
     s_OldStringFound = Text;
 
-    int ii, nbitems, NumOfLibs = NumOfLibraries();
-    if( NumOfLibs == 0 )
+    if( CMP_LIBRARY::GetLibraryCount() == 0 )
     {
-        DisplayError( this, _( "No libraries are loaded" ) );
-        Close(); return;
+        DisplayError( this, _( "No component libraries are loaded." ) );
+        Close();
+        return;
     }
 
-    ListNames = GetLibNames();
+    int nbitemsFound = 0;
 
-    nbitems = 0;
-    for( ii = 0; ii < NumOfLibs; ii++ )    /* Recherche de la librairie */
+    BOOST_FOREACH( CMP_LIBRARY& lib, CMP_LIBRARY::GetLibraryList() )
     {
-        bool IsLibCache;
-        Lib = FindLibrary( ListNames[ii] );
-        if( Lib == NULL )
-            break;
-        if( Lib->m_Name.Contains( wxT( ".cache" ) ) )
-            IsLibCache = TRUE;
-        else
-            IsLibCache = FALSE;
-        LibEntry = (EDA_LibComponentStruct*) PQFirst( &Lib->m_Entries, FALSE );
-        while( LibEntry )
+        nameList.Clear();
+        lib.SearchEntryNames( nameList, Text );
+
+        if( nameList.IsEmpty() )
+            continue;
+
+        nbitemsFound += nameList.GetCount();
+
+        if( !lib.IsCache() )
+            FoundInLib = true;
+
+        for( size_t i = 0; i < nameList.GetCount(); i++ )
         {
-            if( WildCompareString( Text, LibEntry->m_Name.m_Text, FALSE ) )
-            {
-                nbitems++;
-                if( !IsLibCache )
-                    FoundInLib = TRUE;
-                if( !FindList.IsEmpty() )
-                    FindList += wxT( "\n" );
-                FindList << _( "Found " )
-                + LibEntry->m_Name.m_Text
-                + _( " in lib " ) + Lib->m_Name;
-            }
-            LibEntry = (EDA_LibComponentStruct*) PQNext( Lib->m_Entries, LibEntry, NULL );
+            if( !FindList.IsEmpty() )
+                FindList += wxT( "\n" );
+            FindList << _( "Found " ) + nameList[i] + _( " in library " )
+                + lib.GetName();
         }
     }
 
-    free( ListNames );
-
     if( !FoundInLib )
     {
-        if( nbitems )
+        if( nbitemsFound )
             FindList = wxT( "\n" ) + Text + _( " found only in cache" );
         else
             FindList = Text + _( " not found" );
@@ -692,59 +698,49 @@ void WinEDA_FindFrame::LocatePartInLibs( wxCommandEvent& event )
             FindList.Empty();
             ExploreAllLibraries( Text, FindList );
             if( FindList.IsEmpty() )
-                DisplayInfo( this, _( "Nothing found" ) );
+                DisplayInfoMessage( this, _( "Nothing found" ) );
             else
-                DisplayInfo( this, FindList );
+                DisplayInfoMessage( this, FindList );
         }
     }
     else
-        DisplayInfo( this, FindList );
+    {
+        DisplayInfoMessage( this, FindList );
+    }
 
     Close();
 }
 
 
-/***************************************************************************************/
-int WinEDA_FindFrame::ExploreAllLibraries( const wxString& wildmask, wxString& FindList )
-/***************************************************************************************/
+int WinEDA_FindFrame::ExploreAllLibraries( const wxString& wildmask,
+                                           wxString&       FindList )
 {
     wxString FullFileName;
     FILE*    file;
     int      nbitems = 0, LineNum = 0;
     char     Line[2048], * name;
+    wxString path;
 
-    FullFileName = MakeFileName( g_RealLibDirBuffer, wxT( "*" ), g_LibExtBuffer );
-
-    FullFileName = wxFindFirstFile( FullFileName );
-    while( !FullFileName.IsEmpty() )
+    for( unsigned ii = 0; ii < wxGetApp().GetLibraryPathList().GetCount(); ii++ )
     {
-        file = wxFopen( FullFileName, wxT( "rt" ) );
-        if( file == NULL )
-            continue;
+        path = wxGetApp().GetLibraryPathList()[ii] + STRING_DIR_SEP;
+        FullFileName = wxFindFirstFile( path + wxT( "*." ) +
+                                        CompLibFileExtension );
 
-        while( GetLine( file, Line, &LineNum, sizeof(Line) ) )
+        while( !FullFileName.IsEmpty() )
         {
-            if( strnicmp( Line, "DEF", 3 ) == 0 )
+            file = wxFopen( FullFileName, wxT( "rt" ) );
+            if( file == NULL )
+                continue;
+
+            while( GetLine( file, Line, &LineNum, sizeof(Line) ) )
             {
-                /* Read one DEF part from library: DEF 74LS00 U 0 30 Y Y 4 0 N */
-                strtok( Line, " \t\r\n" );
-                name = strtok( NULL, " \t\r\n" );
-                wxString st_name = CONV_FROM_UTF8( name );
-                if( WildCompareString( wildmask, st_name, FALSE ) )
-                {
-                    nbitems++;
-                    if( !FindList.IsEmpty() )
-                        FindList += wxT( "\n" );
-                    FindList << _( "Found " ) << CONV_FROM_UTF8( name )
-                             << _( " in lib " ) << FullFileName;
-                }
-            }
-            else if( strnicmp( Line, "ALIAS", 5 ) == 0 )
-            {
-                /* Read one ALIAS part from library: ALIAS 74HC00 74HCT00 7400 74LS37 */
-                strtok( Line, " \t\r\n" );
-                while( ( name = strtok( NULL, " \t\r\n" ) ) != NULL )
+                if( strnicmp( Line, "DEF", 3 ) == 0 )
                 {
+                    /* Read one DEF part from library:
+                     * DEF 74LS00 U 0 30 Y Y 4 0 N */
+                    strtok( Line, " \t\r\n" );
+                    name = strtok( NULL, " \t\r\n" );
                     wxString st_name = CONV_FROM_UTF8( name );
                     if( WildCompareString( wildmask, st_name, FALSE ) )
                     {
@@ -755,11 +751,29 @@ int WinEDA_FindFrame::ExploreAllLibraries( const wxString& wildmask, wxString& F
                                  << _( " in lib " ) << FullFileName;
                     }
                 }
+                else if( strnicmp( Line, "ALIAS", 5 ) == 0 )
+                {
+                    /* Read one ALIAS part from library:
+                     * ALIAS 74HC00 74HCT00 7400 74LS37 */
+                    strtok( Line, " \t\r\n" );
+                    while( ( name = strtok( NULL, " \t\r\n" ) ) != NULL )
+                    {
+                        wxString st_name = CONV_FROM_UTF8( name );
+                        if( WildCompareString( wildmask, st_name, FALSE ) )
+                        {
+                            nbitems++;
+                            if( !FindList.IsEmpty() )
+                                FindList += wxT( "\n" );
+                            FindList << _( "Found " ) << CONV_FROM_UTF8( name )
+                                     << _( " in lib " ) << FullFileName;
+                        }
+                    }
+                }
             }
-        }
 
-        fclose( file );
-        FullFileName = wxFindNextFile();
+            fclose( file );
+            FullFileName = wxFindNextFile();
+        }
     }
 
     return nbitems;
