@@ -1,8 +1,6 @@
-/***************************************/
-/* Gestion de la LIBRAIRIE des MODULES */
-/***************************************/
-
-/* Fichier LIBRAIRI.CPP */
+/****************************************/
+/* Manage module (footprint) libraries. */
+/****************************************/
 
 #include "fctsys.h"
 #include "appl_wxstruct.h"
@@ -12,47 +10,48 @@
 #include "kicad_string.h"
 #include "gestfich.h"
 #include "pcbnew.h"
+#include "wxPcbStruct.h"
 #include "protos.h"
 
 /*
- *  Format de l'entete de la Librairie:
- *  chaine ENTETE-LIBRAIRIE date-heure
- *  $INDEX
- *  liste des noms modules ( 1 nom par ligne)
- *  $EndINDEX
- *  liste des descriptions des Modules
- *  $EndLIBRARY
+ * Module library header format:
+ * Array LIBRARY HEADER-datetime
+ * $INDEX
+ * List of modules names (1 name per line)
+ * $EndIndex
+ * List of descriptions of Modules
+ * $EndLIBRARY
  */
-
-#define OLD_EXT                    wxT( ".bak" )
-#define FILETMP_EXT                wxT( ".$$$" )
+#define BACKUP_EXT                 wxT( "bak" )
+#define FILETMP_EXT                wxT( "$$$" )
 #define EXPORT_IMPORT_LASTPATH_KEY wxT( "import_last_path" )
 
-/* Fonctions locales */
+const wxString        ModExportFileExtension( wxT( "emp" ) );
+
+static const wxString ModExportFileWildcard(
+    _( "Kicad foot print export files (*.emp)|*.emp" ) );
+
+
 static bool CreateDocLibrary( const wxString& LibName );
 
-/*********************************************************/
-MODULE* WinEDA_ModuleEditFrame::Import_Module( wxDC* DC )
-/*********************************************************/
 
 /**
  * Function Import_Module
  * Read a file containing only one footprint.
  * Used to import (after exporting) a footprint
- * Exported files  have the standart ext .emp
+ * Exported files  have the standard ext .emp
  * This is the same format as .mod files but restricted to only one footprint
  * The import function can also read gpcb footprint file, in Newlib format
  * (One footprint per file, Newlib files have no special ext.)
  * @param DC = Current Device Context (can be NULL)
  */
+MODULE* WinEDA_ModuleEditFrame::Import_Module( wxDC* DC )
 {
     int       NbLine = 0;
     char      Line[1024];
-    wxString  CmpFullFileName;
-    FILE*     dest;
+    FILE*     file;
     MODULE*   module = NULL;
     bool      Footprint_Is_GPCB_Format = false;
-    wxString  mask = wxT( "*.*;" ); mask += EXT_CMP_MASK;
 
     wxString  LastOpenedPathForLoading;
     wxConfig* Config = wxGetApp().m_EDA_Config;
@@ -60,43 +59,42 @@ MODULE* WinEDA_ModuleEditFrame::Import_Module( wxDC* DC )
     if( Config )
         Config->Read( EXPORT_IMPORT_LASTPATH_KEY, &LastOpenedPathForLoading );
 
-    /* Lecture Fichier module */
-    CmpFullFileName = EDA_FileSelector( _( "Import Module:" ),
-                                        LastOpenedPathForLoading,   /* Chemin par defaut */
-                                        wxEmptyString,              /* nom fichier par defaut */
-                                        wxEmptyString,              /* extension par defaut */
-                                        mask,                       /* Masque d'affichage */
-                                        this,
-                                        wxFD_OPEN,
-                                        true
-                                        );
+    wxFileDialog dlg( this, _( "Import Footprint Module" ),
+                      LastOpenedPathForLoading, wxEmptyString,
+                      ModExportFileWildcard, wxFD_OPEN | wxFD_FILE_MUST_EXIST );
 
-    if( CmpFullFileName == wxEmptyString )
+    if( dlg.ShowModal() == wxID_CANCEL )
         return NULL;
 
-    if( ( dest = wxFopen( CmpFullFileName, wxT( "rt" ) ) ) == NULL )
+    file = wxFopen( dlg.GetPath(), wxT( "rt" ) );
+
+    if( file == NULL )
     {
         wxString msg;
-        msg.Printf( _( "File <%s> not found" ), CmpFullFileName.GetData() );
+        msg.Printf( _( "File <%s> not found" ), GetChars( dlg.GetPath() ) );
         DisplayError( this, msg );
         return NULL;
     }
 
     if( Config )    // Save file path
     {
-        LastOpenedPathForLoading = wxPathOnly( CmpFullFileName );
+        LastOpenedPathForLoading = wxPathOnly( dlg.GetPath() );
         Config->Write( EXPORT_IMPORT_LASTPATH_KEY, LastOpenedPathForLoading );
     }
 
+    // Switch the locale to standard C (needed to print floating point numbers
+    // like 1.3)
+    SetLocaleTo_C_standard();
+
     /* Read header and test file type */
-    GetLine( dest, Line, &NbLine );
+    GetLine( file, Line, &NbLine );
     if( strnicmp( Line, ENTETE_LIBRAIRIE, L_ENTETE_LIB ) != 0 )
     {
         if( strnicmp( Line, "Element", 7 ) == 0 )
             Footprint_Is_GPCB_Format = true;
         else
         {
-            fclose( dest );
+            fclose( file );
             DisplayError( this, _( "Not a module file" ) );
             return NULL;
         }
@@ -105,7 +103,7 @@ MODULE* WinEDA_ModuleEditFrame::Import_Module( wxDC* DC )
     /* Read file: Search the description starting line (skip lib header)*/
     if( !Footprint_Is_GPCB_Format )
     {
-        while( GetLine( dest, Line, &NbLine ) != NULL )
+        while( GetLine( file, Line, &NbLine ) != NULL )
         {
             if( strnicmp( Line, "$MODULE", 7 ) == 0 )
                 break;
@@ -116,148 +114,144 @@ MODULE* WinEDA_ModuleEditFrame::Import_Module( wxDC* DC )
 
     if( Footprint_Is_GPCB_Format )
     {
-        fclose( dest );
-        module->Read_GPCB_Descr( CmpFullFileName );
+        fclose( file );
+        module->Read_GPCB_Descr( dlg.GetPath() );
     }
     else
     {
-        module->ReadDescr( dest, &NbLine );
-        fclose( dest );
+        module->ReadDescr( file, &NbLine );
+        fclose( file );
     }
+    SetLocaleTo_Default();       // revert to the current locale
 
     /* Insert footprint in list*/
     GetBoard()->Add( module );
 
     /* Display info : */
-    module->Display_Infos( this );
+    module->DisplayInfo( this );
     Place_Module( module, DC );
     GetBoard()->m_Status_Pcb = 0;
-    build_liste_pads();
+    GetBoard()->m_NetInfo->BuildListOfNets();
 
     return module;
 }
 
 
-/************************************************************************/
-void WinEDA_ModuleEditFrame::Export_Module( MODULE* ptmod, bool createlib )
-/************************************************************************/
-
 /**
  * Function Export_Module
  * Create a file containing only one footprint.
  * Used to export a footprint
- * Exported files  have the standart ext .emp
+ * Exported files  have the standard ext .emp
  * This is the same format as .mod files but restricted to only one footprint
- * So Create a new lib (which will contains one module) and export a footprint is basically the same thing
+ * So Create a new lib (which will contains one module) and export a footprint
+ * is basically the same thing
  * @param DC = Current Device Context (can be NULL)
  * @param createlib : true = use default lib path to create lib
- *                    false = use current path or last used path to export footprint
+ *                    false = use current path or last used path to export
+ * footprint
  */
+void WinEDA_ModuleEditFrame::Export_Module( MODULE* ptmod, bool createlib )
 {
-    wxString  FullFileName, Mask( wxT( "*" ) );
-    char      Line[1025];
-    FILE*     dest;
-    wxString  msg, path;
-    wxConfig* Config = wxGetApp().m_EDA_Config;
+    wxFileName fn;
+    char       Line[1025];
+    FILE*      file;
+    wxString   msg, path, title, wildcard;
+    wxConfig*  Config = wxGetApp().m_EDA_Config;
 
     if( ptmod == NULL )
         return;
 
     ptmod->m_LibRef = ptmod->m_Reference->m_Text;
-    FullFileName    = ptmod->m_LibRef;
-    FullFileName   += createlib ? LibExtBuffer : EXT_CMP;
-
-    Mask += createlib ? LibExtBuffer : EXT_CMP;
+    fn.SetName( ptmod->m_LibRef );
+    fn.SetExt( createlib ? ModuleFileExtension : ModExportFileExtension );
 
     if( createlib )
-        path = g_RealLibDirBuffer;
+        path = wxGetApp().ReturnLastVisitedLibraryPath();
     else if( Config )
         Config->Read( EXPORT_IMPORT_LASTPATH_KEY, &path );
 
-    FullFileName = EDA_FileSelector( createlib ? _( "Create lib" ) : _( "Export Module:" ),
-                                     path,                                  /* Chemin par defaut */
-                                     FullFileName,                          /* nom fichier par defaut */
-                                     createlib ? LibExtBuffer : EXT_CMP,    /* extension par defaut */
-                                     Mask,                                  /* Masque d'affichage */
-                                     this,
-                                     wxFD_SAVE,
-                                     true
-                                     );
+    fn.SetPath( path );
+    title    = createlib ? _( "Create New Library" ) : _( "Export Module" );
+    wildcard = createlib ?  ModuleFileWildcard : ModExportFileWildcard;
+    wxFileDialog dlg( this, msg, fn.GetPath(), fn.GetFullName(), wildcard,
+                      wxFD_SAVE | wxFD_OVERWRITE_PROMPT );
 
-    if( FullFileName.IsEmpty() )
+    if( dlg.ShowModal() == wxID_CANCEL )
         return;
 
-    if( wxFileExists( FullFileName ) )
-    {
-        msg.Printf( _( "File %s exists, OK to replace ?" ),
-                   FullFileName.GetData() );
-        if( !IsOK( this, msg ) )
-            return;
-    }
+    fn = dlg.GetPath();
+    wxGetApp().SaveLastVisitedLibraryPath( fn.GetPath() );
 
-    /* Generation du fichier Empreinte */
-    if( ( dest = wxFopen( FullFileName, wxT( "wt" ) ) ) == NULL )
+    if( ( file = wxFopen( fn.GetFullPath(), wxT( "wt" ) ) ) == NULL )
     {
-        msg.Printf( _( "Unable to create <%s>" ), FullFileName.GetData() );
+        msg.Printf( _( "Unable to create <%s>" ), GetChars( fn.GetFullPath() ) );
         DisplayError( this, msg );
         return;
     }
 
     if( !createlib && Config )  // Save file path
     {
-        path = wxPathOnly( FullFileName );
-        Config->Write( EXPORT_IMPORT_LASTPATH_KEY, path );
+        Config->Write( EXPORT_IMPORT_LASTPATH_KEY, fn.GetPath() );
     }
 
-    fprintf( dest, "%s  %s\n", ENTETE_LIBRAIRIE, DateAndTime( Line ) );
-    fputs( "$INDEX\n", dest );
+    // Switch the locale to standard C (needed to read floating point numbers
+    // like 1.3)
+    SetLocaleTo_C_standard();
 
-    fprintf( dest, "%s\n", CONV_TO_UTF8( ptmod->m_LibRef ) );
-    fputs( "$EndINDEX\n", dest );
+    fprintf( file, "%s  %s\n", ENTETE_LIBRAIRIE, DateAndTime( Line ) );
+    fputs( "$INDEX\n", file );
 
-    GetBoard()->m_Modules->Save( dest );
+    fprintf( file, "%s\n", CONV_TO_UTF8( ptmod->m_LibRef ) );
+    fputs( "$EndINDEX\n", file );
 
-    fputs( "$EndLIBRARY\n", dest );
-    fclose( dest );
-    msg.Printf( _( "Module exported in file <%s>" ), FullFileName.GetData() );
-    DisplayInfo( this, msg );
+    GetBoard()->m_Modules->Save( file );
+
+    fputs( "$EndLIBRARY\n", file );
+    fclose( file );
+    SetLocaleTo_Default();       // revert to the current locale
+
+    msg.Printf( _( "Module exported in file <%s>" ),
+                GetChars( fn.GetFullPath() ) );
+    DisplayInfoMessage( this, msg );
 }
 
 
-/**********************************************************/
-void WinEDA_ModuleEditFrame::Delete_Module_In_Library( const
-                                                       wxString& libname )
-/**********************************************************/
+void WinEDA_ModuleEditFrame::Delete_Module_In_Library(
+    const wxString& aLibname )
 {
-    int      ii, NoFound = 1, LineNum = 0;
-    char     Line[1024], Name[256];
-    wxString NewLib, OldLib;
-    FILE*    dest, * lib_module;
-    wxString CmpName, msg;
+    wxFileName newFileName;
+    wxFileName oldFileName;
+    int        ii, NoFound = 1, LineNum = 0;
+    char       Line[1024], Name[256];
+    FILE*      dest, * lib_module;
+    wxString   CmpName, msg;
 
-    /* Demande du nom du composant a supprimer */
-    CmpName = Select_1_Module_From_List( this, libname, wxEmptyString, wxEmptyString );
+    CmpName = Select_1_Module_From_List( this,
+                                         aLibname,
+                                         wxEmptyString,
+                                         wxEmptyString );
     if( CmpName == wxEmptyString )
         return;
 
     /* Confirmation */
     msg.Printf( _( "Ok to delete module %s in library %s" ),
-               CmpName.GetData(), libname.GetData() );
+               GetChars( CmpName ), GetChars( aLibname ) );
     if( !IsOK( this, msg ) )
         return;
 
-    OldLib = libname;
+    oldFileName = aLibname;
 
-    if( ( lib_module = wxFopen( OldLib, wxT( "rt" ) ) )  == NULL )
+    if( ( lib_module = wxFopen( oldFileName.GetFullPath(),
+                               wxT( "rt" ) ) )  == NULL )
     {
         wxString msg;
-        msg = _( "Library " ) + OldLib + _( " not found" );
+        msg = _( "Library " ) + oldFileName.GetFullPath() + _( " not found" );
         DisplayError( this, msg );
         return;
     }
 
 
-    /* lecture entete */
+    /* Read header. */
     GetLine( lib_module, Line, &LineNum );
 
     if( strnicmp( Line, ENTETE_LIBRAIRIE, L_ENTETE_LIB ) != 0 )
@@ -267,7 +261,7 @@ void WinEDA_ModuleEditFrame::Delete_Module_In_Library( const
         return;
     }
 
-    /* lecture des nom des composants  */
+    /* Read module names.  */
     while( GetLine( lib_module, Line, &LineNum ) )
     {
         if( strnicmp( Line, "$INDEX", 6 ) == 0 )
@@ -276,7 +270,7 @@ void WinEDA_ModuleEditFrame::Delete_Module_In_Library( const
             {
                 StrPurge( Line );
                 msg = CONV_FROM_UTF8( Line );
-                if( CmpName.CmpNoCase( msg ) == 0 ) /* composant trouve */
+                if( CmpName.CmpNoCase( msg ) == 0 ) /* New module? */
                 {
                     NoFound = 0; break;
                 }
@@ -291,30 +285,33 @@ void WinEDA_ModuleEditFrame::Delete_Module_In_Library( const
     if( NoFound )
     {
         fclose( lib_module );
-        msg.Printf( _( "Module [%s] not found" ), CmpName.GetData() );
+        msg.Printf( _( "Module [%s] not found" ), GetChars( CmpName ) );
         DisplayError( this, msg );
         return;
     }
 
-    /* Creation de la nouvelle librairie */
-    NewLib = OldLib;
-    ChangeFileNameExt( NewLib, FILETMP_EXT );
-    if( ( dest = wxFopen( NewLib, wxT( "wt" ) ) ) == NULL )
+    /* Create new library. */
+    newFileName = oldFileName;
+    newFileName.SetExt( FILETMP_EXT );
+
+    if( ( dest = wxFopen( newFileName.GetFullPath(), wxT( "wt" ) ) ) == NULL )
     {
         fclose( lib_module );
         wxString msg;
-        msg = _( "Unable to create " ) + NewLib;
+        msg = _( "Unable to create " ) + newFileName.GetFullPath();
         DisplayError( this, msg );
         return;
     }
 
     wxBeginBusyCursor();
 
-    /* Creation de l'entete avec nouvelle date */
+    /* Create header with new date. */
     fprintf( dest, ENTETE_LIBRAIRIE );
     fprintf( dest, "  %s\n$INDEX\n", DateAndTime( Line ) );
 
-    fseek( lib_module, 0, 0 ); GetLine( lib_module, Line, &ii );
+    fseek( lib_module, 0, 0 );
+    GetLine( lib_module, Line, &ii );
+
     while( GetLine( lib_module, Line, &ii ) )
     {
         if( strnicmp( Line, "$M", 2 ) == 0 )
@@ -337,7 +334,7 @@ void WinEDA_ModuleEditFrame::Delete_Module_In_Library( const
 
     fprintf( dest, "$EndINDEX\n" );
 
-    /* Copie des modules */
+    /* Copy modules. */
     while( GetLine( lib_module, Line, &LineNum ) )
     {
         StrPurge( Line );
@@ -347,7 +344,7 @@ void WinEDA_ModuleEditFrame::Delete_Module_In_Library( const
             msg = CONV_FROM_UTF8( Name );
             if( msg.CmpNoCase( CmpName ) == 0 )
             {
-                /* suppression ancien module */
+                /* Delete old module. */
                 while( GetLine( lib_module, Line, &LineNum ) )
                 {
                     if( strnicmp( Line, "$EndMODULE", 9 ) == 0 )
@@ -365,78 +362,76 @@ void WinEDA_ModuleEditFrame::Delete_Module_In_Library( const
 
     wxEndBusyCursor();
 
-    /* Le fichier ancienne librairie est renommee en .bak */
-    wxString BakFilename = OldLib;
-    ChangeFileNameExt( BakFilename, OLD_EXT );
+    /* The old library file is renamed .bak */
+    wxFileName backupFileName = oldFileName;
+    backupFileName.SetExt( BACKUP_EXT );
 
-    if( wxFileExists( BakFilename ) )
-        wxRemoveFile( BakFilename );
+    if( backupFileName.FileExists() )
+        wxRemoveFile( backupFileName.GetFullPath() );
 
-    if( !wxRenameFile( OldLib, BakFilename ) )
+    if( !wxRenameFile( oldFileName.GetFullPath(),
+                      backupFileName.GetFullPath() ) )
     {
         DisplayError( this, wxT( "Librairi.cpp: rename .bak err" ) );
         return;
     }
 
-    /* Le fichier temporaire est renommee comme l'ancienne Lib */
-    if( !wxRenameFile( NewLib, OldLib ) )
+    /* The temporary file is renamed as the previous library. */
+    if( !wxRenameFile( newFileName.GetFullPath(), oldFileName.GetFullPath() ) )
     {
         DisplayError( this, wxT( "Librairi.cpp: rename err 2" ) );
         return;
     }
 
-    msg.Printf( _( "Component %s deleted in library %s" ), CmpName.GetData(), OldLib.GetData() );
+    msg.Printf( _( "Component %s deleted in library %s" ), GetChars( CmpName ),
+               GetChars( oldFileName.GetFullPath() ) );
     Affiche_Message( msg );
 
-    CreateDocLibrary( OldLib );
+    CreateDocLibrary( oldFileName.GetFullPath() );
 }
 
 
-/***********************************************************************/
+/*
+ * Save library:
+ * All new modules (ie modules not already exist (if NewModulesOnly == true)
+ * all modules (if NewModulesOnly == FALSE)
+ */
 void WinEDA_BasePcbFrame::Archive_Modules( const wxString& LibName,
                                            bool            NewModulesOnly )
-/***********************************************************************/
-
-/*
- *  Sauve en Librairie:
- *  tous les nouveaux modules ( c.a.d. les modules
- *      n'existant pas deja (si NewModulesOnly == true)
- *  tous les modules (si NewModulesOnly == FALSE)
- */
 {
     int      ii, NbModules = 0;
     float    Pas;
     MODULE*  Module;
-    wxString FullFileName = LibName;
+    wxString fileName = LibName, path;
 
     if( GetBoard()->m_Modules == NULL )
     {
-        DisplayInfo( this, _( " No modules to archive!" ) );
+        DisplayInfoMessage( this, _( " No modules to archive!" ) );
         return;
     }
 
-    if( FullFileName.IsEmpty() )
+    path = wxGetApp().ReturnLastVisitedLibraryPath();
+    if( LibName.IsEmpty() )
     {
-        wxString Mask = wxT( "*" ) + LibExtBuffer;
-        FullFileName = EDA_FileSelector( _( "Library" ),
-                                         g_RealLibDirBuffer,    /* Chemin par defaut */
-                                         FullFileName,          /* nom fichier par defaut */
-                                         LibExtBuffer,          /* extension par defaut */
-                                         Mask,                  /* Masque d'affichage */
-                                         this,
-                                         wxFD_SAVE,
-                                         true
-                                         );
+        wxFileDialog dlg( this, _( "Library" ), path,
+                          wxEmptyString, ModuleFileWildcard,
+                          wxFD_SAVE | wxFD_OVERWRITE_PROMPT );
 
-        if( FullFileName.IsEmpty() )
+        if( dlg.ShowModal() == wxID_CANCEL )
             return;
+
+        fileName = dlg.GetPath();
     }
 
-    bool file_exists = wxFileExists( FullFileName );
+    wxFileName fn( fileName );
+    wxGetApp().SaveLastVisitedLibraryPath( fn.GetPath() );
+    bool       file_exists = wxFileExists( fileName );
+
     if( !NewModulesOnly && file_exists )
     {
         wxString msg;
-        msg.Printf( _( "File %s exists, OK to replace ?" ), FullFileName.GetData() );
+        msg.Printf( _( "File %s exists, OK to replace ?" ),
+                    GetChars( fileName ) );
         if( !IsOK( this, msg ) )
             return;
     }
@@ -447,9 +442,9 @@ void WinEDA_BasePcbFrame::Archive_Modules( const wxString& LibName,
     if( !NewModulesOnly || !file_exists )
     {
         FILE* lib_module;
-        if( ( lib_module = wxFopen( FullFileName, wxT( "w+t" ) ) )  == NULL )
+        if( ( lib_module = wxFopen( fileName, wxT( "w+t" ) ) )  == NULL )
         {
-            wxString msg = _( "Unable to create " ) + FullFileName;
+            wxString msg = _( "Unable to create " ) + fileName;
             DisplayError( this, msg );
             return;
         }
@@ -461,7 +456,7 @@ void WinEDA_BasePcbFrame::Archive_Modules( const wxString& LibName,
         fclose( lib_module );
     }
 
-    /* Calcul du nombre de modules */
+    /* Calculate the number of modules. */
     Module = (MODULE*) GetBoard()->m_Modules;
     for( ; Module != NULL; Module = (MODULE*) Module->Next() )
         NbModules++;
@@ -472,53 +467,56 @@ void WinEDA_BasePcbFrame::Archive_Modules( const wxString& LibName,
     Module = (MODULE*) GetBoard()->m_Modules;
     for( ii = 1; Module != NULL; ii++, Module = (MODULE*) Module->Next() )
     {
-        if( Save_Module_In_Library( FullFileName, Module,
-                                    NewModulesOnly ? FALSE : true, FALSE, false ) == 0 )
+        if( Save_Module_In_Library( fileName, Module,
+                                    NewModulesOnly ? FALSE : true,
+                                    FALSE, false ) == 0 )
             break;
         DisplayActivity( (int) ( ii * Pas ), wxEmptyString );
-        /* Tst demande d'arret de sauvegarde ( key ESCAPE actionnee ) */
+        /* Check for request to stop backup (ESCAPE key actuated) */
         if( DrawPanel->m_AbortRequest )
             break;
     }
 
-    CreateDocLibrary( FullFileName );
+    CreateDocLibrary( fileName );
 }
 
 
-/*****************************************************************/
-int WinEDA_BasePcbFrame::Save_Module_In_Library( const wxString& aLibName,
-                                                 MODULE* aModule, bool aOverwrite,
-                                                 bool aDisplayDialog, bool aCreateDocFile )
-/*****************************************************************/
-
 /** Function Save_Module_In_Library
  *  Save in an existing library a given footprint
  * @param aLibName = name of the library to use
  * @param aModule = the given footprint
- * @param aOverwrite = true to overwrite an existing footprint, false to abort an existing footprint is found
- * @param aDisplayDialog = true to display a dialog to enter or confirm the footprint name
+ * @param aOverwrite = true to overwrite an existing footprint, false to abort
+ * an existing footprint is found
+ * @param aDisplayDialog = true to display a dialog to enter or confirm the
+ * footprint name
  * @param aCreateDocFile = true to creates the associated doc file
  * @return :  1 if OK, 0 if abort
  */
+int WinEDA_BasePcbFrame::Save_Module_In_Library( const wxString& aLibName,
+                                                 MODULE*         aModule,
+                                                 bool            aOverwrite,
+                                                 bool            aDisplayDialog,
+                                                 bool            aCreateDocFile )
 {
-    int      newmodule, end;
-    int      LineNum = 0, tmp;
-    char     Name[256], Line[1024];
-    wxString Name_Cmp;
-    wxString NewLib, OldLib, msg;
-    FILE*    lib_module, * dest;
-    bool     added = true;
+    wxFileName oldFileName;
+    wxFileName newFileName;
+    int        newmodule, end;
+    int        LineNum = 0, tmp;
+    char       Name[256], Line[1024];
+    wxString   Name_Cmp;
+    wxString   msg;
+    FILE*      lib_module, * dest;
+    bool       added = true;
 
-    aModule->Display_Infos( this );
+    aModule->DisplayInfo( this );
 
     if( !wxFileExists( aLibName ) )
     {
-        msg.Printf( _( "Library %s not found" ), aLibName.GetData() );
+        msg.Printf( _( "Library %s not found" ), GetChars( aLibName ) );
         DisplayError( this, msg );
         return 0;
     }
 
-
     /* Ask for the footprint name in lib */
     Name_Cmp = aModule->m_LibRef;
 
@@ -534,22 +532,23 @@ int WinEDA_BasePcbFrame::Save_Module_In_Library( const wxString& aLibName,
 
     if( ( lib_module = wxFopen( aLibName, wxT( "rt" ) ) ) == NULL )
     {
-        msg.Printf( _( "Unable to open %s" ), aLibName.GetData() );
+        msg.Printf( _( "Unable to open %s" ), GetChars( aLibName ) );
         DisplayError( this, msg );
         return 0;
     }
 
-    /* lRead library file : library header */
+    /* Read library file : library header */
     GetLine( lib_module, Line, &LineNum );
     if( strnicmp( Line, ENTETE_LIBRAIRIE, L_ENTETE_LIB ) != 0 )
     {
         fclose( lib_module );
-        msg.Printf( _( "File %s is not a eeschema library" ), aLibName.GetData() );
+        msg.Printf( _( "File %s is not a eeschema library" ),
+                   GetChars( aLibName ) );
         DisplayError( this, msg );
         return 0;
     }
 
-    /* Reaf footprints in lib: - search for an existing footprint */
+    /* Read footprints in lib: - search for an existing footprint */
     newmodule = 1; end = 0;
     while( !end && GetLine( lib_module, Line, &LineNum ) )
     {
@@ -567,7 +566,8 @@ int WinEDA_BasePcbFrame::Save_Module_In_Library( const wxString& aLibName,
 
             StrPurge( Line );
             msg = CONV_FROM_UTF8( Line );
-            if( Name_Cmp.CmpNoCase( msg ) == 0 ) /* an existing footprint is found */
+            if( Name_Cmp.CmpNoCase( msg ) == 0 ) /* an existing footprint is
+                                                  * found */
             {
                 added     = FALSE;
                 newmodule = 0;
@@ -577,7 +577,8 @@ int WinEDA_BasePcbFrame::Save_Module_In_Library( const wxString& aLibName,
                     msg << LineNum;
                     Affiche_Message( msg );
                 }
-                if( !aOverwrite )    /* lDo not save the given footprint: an old one exists */
+                if( !aOverwrite )    /* Do not save the given footprint: an old
+                                      * one exists */
                 {
                     fclose( lib_module ); return 1;
                 }
@@ -596,18 +597,23 @@ int WinEDA_BasePcbFrame::Save_Module_In_Library( const wxString& aLibName,
         return 0;
     }
 
-    NewLib = aLibName;
-    ChangeFileNameExt( NewLib, FILETMP_EXT );
-    if( ( dest = wxFopen( NewLib, wxT( "w+t" ) ) )  == NULL )
+    newFileName = aLibName;
+    newFileName.SetExt( FILETMP_EXT );
+
+    if( ( dest = wxFopen( newFileName.GetFullPath(), wxT( "w+t" ) ) )  == NULL )
     {
         fclose( lib_module );
-        msg = _( "Unable to create " ) + NewLib;
+        msg = _( "Unable to create " ) + newFileName.GetFullPath();
         DisplayError( this, msg );
         return 0;
     }
 
     wxBeginBusyCursor();
 
+    // Switch the locale to standard C (needed to print floating point numbers
+    // like 1.3)
+    SetLocaleTo_C_standard();
+
     /* Create the library header with a new date */
     fprintf( dest, ENTETE_LIBRAIRIE );
     fprintf( dest, "  %s\n$INDEX\n", DateAndTime( Line ) );
@@ -648,7 +654,7 @@ int WinEDA_BasePcbFrame::Save_Module_In_Library( const wxString& aLibName,
             msg = CONV_FROM_UTF8( Name );
             if( msg.CmpNoCase( Name_Cmp ) == 0 )
             {
-                /* skip old footprint descr (delete from the lib)*/
+                /* skip old footprint descr (delete from the lib) */
                 while( GetLine( lib_module, Line, &LineNum ) )
                 {
                     if( strnicmp( Line, "$EndMODULE", 9 ) == 0 )
@@ -668,28 +674,29 @@ int WinEDA_BasePcbFrame::Save_Module_In_Library( const wxString& aLibName,
     aModule->m_TimeStamp = tmp;
 
     fclose( dest );  fclose( lib_module );
+    SetLocaleTo_Default();       // revert to the current locale
 
     wxEndBusyCursor();
 
     /* The old library file is renamed .bak */
-    OldLib = aLibName;
-    ChangeFileNameExt( OldLib, OLD_EXT );
+    oldFileName = aLibName;
+    oldFileName.SetExt( BACKUP_EXT );
 
-    if( wxFileExists( OldLib ) )
-        wxRemoveFile( OldLib );
+    if( oldFileName.FileExists() )
+        wxRemoveFile( oldFileName.GetFullPath() );
 
-    if( !wxRenameFile( aLibName, OldLib ) )
+    if( !wxRenameFile( aLibName, oldFileName.GetFullPath() ) )
         DisplayError( this, wxT( "Librairi.cpp: rename .bak err" ) );
 
     /* The new library file is renamed */
-    if( !wxRenameFile( NewLib, aLibName ) )
+    if( !wxRenameFile( newFileName.GetFullPath(), aLibName ) )
     {
         DisplayError( this, wxT( "Librairi.cpp: rename NewLib err" ) );
         return 0;
     }
 
     /* creates the new .dcm doc file corresponding to the new library */
-    if ( aCreateDocFile )
+    if( aCreateDocFile )
         CreateDocLibrary( aLibName );
 
     if( aDisplayDialog )
@@ -704,27 +711,25 @@ int WinEDA_BasePcbFrame::Save_Module_In_Library( const wxString& aLibName,
 }
 
 
-/************************************************************************************/
-MODULE* WinEDA_BasePcbFrame::Create_1_Module( wxDC* DC, const wxString& module_name )
-/************************************************************************************/
-
-/* Create a new module or footprint : A new module is tartted with 2 texts :
+/* Create a new module or footprint : A new module contains 2 texts :
  *          First = REFERENCE
  *          Second = VALUE: "VAL**"
- *  the new module is added on begining of the linked list of modules
+ *  the new module is added on beginning of the linked list of modules
  */
-
+MODULE* WinEDA_BasePcbFrame::Create_1_Module( wxDC*           DC,
+                                              const wxString& module_name )
 {
     MODULE*  Module;
     wxString Line;
     wxPoint  newpos;
 
-    /* Ask fo the new module reference */
+    /* Ask for the new module reference */
     if( module_name.IsEmpty() )
     {
-        if( Get_Message( _( "Module Reference:" ), _( "Module Creation" ), Line, this ) != 0 )
+        if( Get_Message( _( "Module Reference:" ),
+                         _( "Module Creation" ), Line, this ) != 0 )
         {
-            DisplayInfo( this, _( "No reference, aborted" ) );
+            DisplayInfoMessage( this, _( "No reference, aborted" ) );
             return NULL;
         }
     }
@@ -733,7 +738,8 @@ MODULE* WinEDA_BasePcbFrame::Create_1_Module( wxDC* DC, const wxString& module_n
     Line.Trim( true );
     Line.Trim( FALSE );
 
-    // Creates the new module and add it to the head of the linked list of modules
+    // Creates the new module and add it to the head of the linked list of
+    // modules
     Module = new MODULE( GetBoard() );
 
     GetBoard()->Add( Module );
@@ -758,21 +764,19 @@ MODULE* WinEDA_BasePcbFrame::Create_1_Module( wxDC* DC, const wxString& module_n
 
     Module->SetPosition( wxPoint( 0, 0 ) );
 
-    Module->Display_Infos( this );
+    Module->DisplayInfo( this );
     return Module;
 }
 
 
-/*******************************************************/
 void WinEDA_ModuleEditFrame::Select_Active_Library()
-/*******************************************************/
 {
     if( g_LibName_List.GetCount() == 0 )
         return;
 
     WinEDAListBox* LibListBox = new WinEDAListBox( this, _( "Active Lib:" ),
-                                                  NULL, m_CurrentLib, NULL,
-                                                  wxColour( 200, 200, 255 ) );
+                                                   NULL, m_CurrentLib, NULL,
+                                                   wxColour( 200, 200, 255 ) );
 
     LibListBox->InsertItems( g_LibName_List );
 
@@ -787,9 +791,7 @@ void WinEDA_ModuleEditFrame::Select_Active_Library()
 }
 
 
-/**********************************************************************/
 int WinEDA_ModuleEditFrame::Create_Librairie( const wxString& LibName )
-/**********************************************************************/
 {
     FILE*    lib_module;
     wxString msg;
@@ -809,7 +811,7 @@ int WinEDA_ModuleEditFrame::Create_Librairie( const wxString& LibName )
         return -1;
     }
 
-    /* Ecriture de l'entete de la nouvelle librairie */
+    /* Write the header of the new library. */
     if( fprintf( lib_module, ENTETE_LIBRAIRIE ) == 0 )
     {
         msg = _( "Create error " ) + LibName;
@@ -826,28 +828,25 @@ int WinEDA_ModuleEditFrame::Create_Librairie( const wxString& LibName )
 }
 
 
-/******************************************************/
-static bool CreateDocLibrary( const wxString& LibName )
-/*****************************************************/
-
-/* Creation du fichier .dcm associe a la librairie LibName
- *  (full file name)
+/* Synch. Dcm combines a library Libname
+ * (Full file name)
  */
+static bool CreateDocLibrary( const wxString& LibName )
 {
-    char     Line[1024];
-    char     cbuf[256];
-    wxString Name, Doc, KeyWord;
-    wxString LibDocName;
-    FILE*    LibMod, * LibDoc;
+    char       Line[1024];
+    char       cbuf[256];
+    wxString   Name, Doc, KeyWord;
+    wxFileName fn;
+    FILE*      LibMod, * LibDoc;
 
-    LibDocName = LibName;
-    ChangeFileNameExt( LibDocName, EXT_DOC );
+    fn = LibName;
+    fn.SetExt( EXT_DOC );
 
     LibMod = wxFopen( LibName, wxT( "rt" ) );
     if( LibMod == NULL )
         return FALSE;
 
-    /* lecture entete librairie*/
+    /* Read library header. */
     GetLine( LibMod, Line, NULL, sizeof(Line) - 1 );
     if( strnicmp( Line, ENTETE_LIBRAIRIE, L_ENTETE_LIB ) != 0 )
     {
@@ -855,7 +854,7 @@ static bool CreateDocLibrary( const wxString& LibName )
         return FALSE;
     }
 
-    LibDoc = wxFopen( LibDocName, wxT( "wt" ) );
+    LibDoc = wxFopen( fn.GetFullPath(), wxT( "wt" ) );
     if( LibDoc == NULL )
     {
         fclose( LibMod );
@@ -864,7 +863,7 @@ static bool CreateDocLibrary( const wxString& LibName )
     fprintf( LibDoc, ENTETE_LIBDOC );
     fprintf( LibDoc, "  %s\n", DateAndTime( cbuf ) );
 
-    /* Lecture de la librairie */
+    /* Read library. */
     Name = Doc = KeyWord = wxEmptyString;
     while( GetLine( LibMod, Line, NULL, sizeof(Line) - 1 ) )
     {
@@ -897,7 +896,9 @@ static bool CreateDocLibrary( const wxString& LibName )
                     Doc = CONV_FROM_UTF8( StrPurge( Line + 3 ) );
             }
 
-            if( (Name != wxEmptyString) && ( (Doc != wxEmptyString) || (KeyWord != wxEmptyString) ) ) /* Generation de la doc du composant */
+            /* Generate the module the documentation. */
+            if( ( Name != wxEmptyString )
+               && ( ( Doc != wxEmptyString ) || ( KeyWord != wxEmptyString ) ) )
             {
                 fprintf( LibDoc, "#\n$MODULE %s\n", CONV_TO_UTF8( Name ) );
                 fprintf( LibDoc, "Li %s\n", CONV_TO_UTF8( Name ) );
@@ -910,7 +911,7 @@ static bool CreateDocLibrary( const wxString& LibName )
                 fprintf( LibDoc, "$EndMODULE\n" );
             }
             Name = Doc = KeyWord = wxEmptyString;
-        }     /* Fin lecture desc 1 module */
+        }     /* End read 1 module */
 
         if( strnicmp( Line, "$INDEX", 6 ) == 0 )
         {
@@ -920,12 +921,10 @@ static bool CreateDocLibrary( const wxString& LibName )
                     break;
             }
 
-            /* Fin Lecture INDEX */
+            /* End read INDEX */
         }
     }
 
-    /* Fin lecture 1 Librairie */
-
     fclose( LibMod );
     fprintf( LibDoc, "#\n$EndLIBDOC\n" );
     fclose( LibDoc );
