@@ -11,8 +11,10 @@
 #include "kicad_string.h"
 #include "gestfich.h"
 #include "get_component_dialog.h"
+#include "appl_wxstruct.h"
 
 #include "pcbnew.h"
+#include "wxPcbStruct.h"
 #include "protos.h"
 
 class ModList
@@ -21,8 +23,7 @@ public:
     ModList* Next;
     wxString m_Name, m_Doc, m_KeyWord;
 
-public:
-    ModList()
+public: ModList()
     {
         Next = NULL;
     }
@@ -33,26 +34,23 @@ public:
     }
 };
 
-/* Fonctions locales */
+
 static void DisplayCmpDoc( wxString& Name );
 static void ReadDocLib( const wxString& ModLibName );
 
-/*****/
 
-/* variables locales */
 static ModList* MList;
 
 
-/***************************************************************************/
-void WinEDA_ModuleEditFrame::Load_Module_Module_From_BOARD( MODULE* Module )
-/***************************************************************************/
+void WinEDA_ModuleEditFrame::Load_Module_From_BOARD( MODULE* Module )
 {
     MODULE* NewModule;
     WinEDA_BasePcbFrame* parent = (WinEDA_BasePcbFrame*) GetParent();
 
     if( Module == NULL )
     {
-        if( parent->GetBoard() == NULL || parent->GetBoard()->m_Modules == NULL )
+        if( parent->GetBoard() == NULL
+            || parent->GetBoard()->m_Modules == NULL )
             return;
 
         Module = Select_1_Module_From_BOARD( parent->GetBoard() );
@@ -76,77 +74,88 @@ void WinEDA_ModuleEditFrame::Load_Module_Module_From_BOARD( MODULE* Module )
 
     Module->m_Flags = 0;
 
-    build_liste_pads();
+    GetBoard()->m_NetInfo->BuildListOfNets();
 
     GetScreen()->m_Curseur.x = GetScreen()->m_Curseur.y = 0;
     Place_Module( Module, NULL );
-    if( Module->GetLayer() != CMP_N )
-        GetBoard()->Change_Side_Module( Module, NULL );
+    if( Module->GetLayer() != LAYER_N_FRONT )
+        Module->Flip( Module->m_Pos );
     Rotate_Module( NULL, Module, 0, FALSE );
     GetScreen()->ClrModify();
     Zoom_Automatique( TRUE );
 }
 
 
-/****************************************************************************/
 MODULE* WinEDA_BasePcbFrame::Load_Module_From_Library( const wxString& library,
                                                        wxDC*           DC )
-/****************************************************************************/
-/* Permet de charger un module directement a partir de la librairie */
 {
-    MODULE*              module;
-    wxPoint              curspos = GetScreen()->m_Curseur;
+    MODULE* module;
+    wxPoint curspos = GetScreen()->m_Curseur;
     wxString             ModuleName, keys;
     static wxArrayString HistoryList;
-    bool AllowWildSeach = TRUE;
+    static wxString      lastCommponentName;
+    bool             AllowWildSeach = TRUE;
 
     /* Ask for a component name or key words */
-    ModuleName = GetComponentName( this, HistoryList, _( "Place module" ), NULL );
-    ModuleName.MakeUpper();
+    WinEDA_SelectCmp dlg( this, GetComponentDialogPosition(), HistoryList,
+                          _( "Place Module" ), false );
+
+    dlg.SetComponentName( lastCommponentName );
+
+    if( dlg.ShowModal() == wxID_CANCEL )
+        return NULL;
+
+    ModuleName = dlg.GetComponentName();
+
     if( ModuleName.IsEmpty() )  /* Cancel command */
     {
         DrawPanel->MouseToCursorSchema();
         return NULL;
     }
 
+    ModuleName.MakeUpper();
 
     if( ModuleName[0] == '=' )   // Selection by keywords
     {
         AllowWildSeach = FALSE;
         keys = ModuleName.AfterFirst( '=' );
-        ModuleName = Select_1_Module_From_List( this, library, wxEmptyString, keys );
+        ModuleName = Select_1_Module_From_List( this, library, wxEmptyString,
+                                                keys );
         if( ModuleName.IsEmpty() )  /* Cancel command */
         {
             DrawPanel->MouseToCursorSchema();
             return NULL;
         }
     }
-    else if( ( ModuleName.Contains( wxT( "?" ) ) ) || ( ModuleName.Contains( wxT( "*" ) ) ) ) // Selection wild card
+    else if( ( ModuleName.Contains( wxT( "?" ) ) )
+            || ( ModuleName.Contains( wxT( "*" ) ) ) )  // Selection wild card
     {
         AllowWildSeach = FALSE;
-        ModuleName = Select_1_Module_From_List( this, library, ModuleName, wxEmptyString );
+        ModuleName     = Select_1_Module_From_List( this, library, ModuleName,
+                                                    wxEmptyString );
         if( ModuleName.IsEmpty() )
         {
             DrawPanel->MouseToCursorSchema();
-            return NULL;    /* annulation de commande */
+            return NULL;    /* Cancel command. */
         }
     }
 
-    module = Get_Librairie_Module( this, library, ModuleName, FALSE );
+    module = Get_Librairie_Module( library, ModuleName, FALSE );
 
-    if( (module == NULL) && AllowWildSeach )    /* Attemp to search with wildcard */
+    if( ( module == NULL ) && AllowWildSeach )    /* Search with wildcard */
     {
         AllowWildSeach = FALSE;
         wxString wildname = wxChar( '*' ) + ModuleName + wxChar( '*' );
         ModuleName = wildname;
-        ModuleName = Select_1_Module_From_List( this, library, ModuleName, wxEmptyString );
+        ModuleName = Select_1_Module_From_List( this, library, ModuleName,
+                                                wxEmptyString );
         if( ModuleName.IsEmpty() )
         {
             DrawPanel->MouseToCursorSchema();
-            return NULL;    /* annulation de commande */
+            return NULL;    /* Cancel command. */
         }
         else
-            module = Get_Librairie_Module( this, library, ModuleName, TRUE );
+            module = Get_Librairie_Module( library, ModuleName, TRUE );
     }
 
     GetScreen()->m_Curseur = curspos;
@@ -154,6 +163,7 @@ MODULE* WinEDA_BasePcbFrame::Load_Module_From_Library( const wxString& library,
 
     if( module )
     {
+        lastCommponentName = ModuleName;
         AddHistoryComponentName( HistoryList, ModuleName );
 
         module->m_Flags     = IS_NEW;
@@ -161,93 +171,125 @@ MODULE* WinEDA_BasePcbFrame::Load_Module_From_Library( const wxString& library,
         module->m_TimeStamp = GetTimeStamp();
         GetBoard()->m_Status_Pcb = 0;
         module->SetPosition( curspos );
-        build_liste_pads();
 
-        module->Draw( DrawPanel, DC, GR_OR );
+        /* TODO: call RecalculateAllTracksNetcode() only if some pads pads have
+         * a netname.
+         * If all pads are not connected (usually the case in module libraries,
+         * rebuild only the pad and list of nets ( faster)
+         */
+
+//        GetBoard()->m_Pcb->m_NetInfo->BuildListOfNets();
+        RecalculateAllTracksNetcode();
+
+        if( DC )
+            module->Draw( DrawPanel, DC, GR_OR );
     }
 
     return module;
 }
 
 
-/*******************************************************************************/
-MODULE* WinEDA_BasePcbFrame::Get_Librairie_Module( wxWindow* winaff,
-                                                   const wxString& library,
-                                                   const wxString& ModuleName, bool show_msg_err )
-/*******************************************************************************/
-
-/*
- *  Analyse les LIBRAIRIES pour trouver le module demande
- *  Si ce module est trouve, le copie en memoire, et le
- *  chaine en fin de liste des modules
- *      - Entree:
- *          name_cmp = nom du module
- *      - Retour:
- *          Pointeur sur le nouveau module.
+/** function Get_Librairie_Module
+ *
+ *  Read active libraries or one library to find and load a given module
+ *  If found the module is linked to the tail of linked list of modules
+ *  @param aLibrary: the full filename of the library to read. If empty,
+ *                    all active libraries are read
+ *  @param aModuleName = module name to load
+ *  @param aDisplayMessageError = true to display an error message if any.
+ *  @return a MODULE * pointer to the new module, or NULL
+ *
  */
+MODULE* WinEDA_BasePcbFrame::Get_Librairie_Module(
+    const wxString& aLibraryFullFilename,
+    const wxString& aModuleName,
+    bool            aDisplayMessageError )
 {
-    int      LineNum, Found = 0;
-    wxString fulllibname;
-    char     Line[512];
-    wxString Name;
-    wxString ComponentName, msg;
-    MODULE*  NewModule;
-    FILE*    lib_module = NULL;
-    unsigned ii;
-
-    ComponentName = ModuleName;
+    int        LineNum, Found = 0;
+    wxFileName fn;
+    char       Line[512];
+    wxString   Name;
+    wxString   msg, tmp;
+    MODULE*    NewModule;
+    FILE*      file = NULL;
+    unsigned   ii;
+    bool       one_lib = aLibraryFullFilename.IsEmpty() ? false : true;
 
     for( ii = 0; ii < g_LibName_List.GetCount(); ii++ )
     {
-        fulllibname = g_LibName_List[ii];
+        if( one_lib )
+            fn = aLibraryFullFilename;
+        else
+            fn = wxFileName( wxEmptyString, g_LibName_List[ii],
+                             ModuleFileExtension );
 
-        /* Calcul du nom complet de la librairie */
-        fulllibname = MakeFileName( g_RealLibDirBuffer, fulllibname, LibExtBuffer );
+        tmp = wxGetApp().FindLibraryPath( fn );
 
-        if( ( lib_module = wxFopen( fulllibname, wxT( "rt" ) ) )  == NULL )
+        if( !tmp )
         {
-            msg.Printf( _( "Library <%s> not found" ), fulllibname.GetData() );
-            Affiche_Message( msg );
+            if( aDisplayMessageError )
+            {
+                msg.Printf( _( "PCB footprint library file <%s> not found in search paths." ),
+                            GetChars( fn.GetFullName() ) );
+                wxMessageBox( msg, _( "Library Load Error" ),
+                              wxOK | wxICON_ERROR, this );
+            }
             continue;
         }
 
-        msg.Printf( _( "Scan Lib: %s" ), fulllibname.GetData() );
+        file = wxFopen( tmp, wxT( "rt" ) );
+
+        if( file == NULL )
+        {
+            msg.Printf( _( "Could not open PCB footprint library file <%s>." ),
+                        GetChars( tmp ) );
+            wxMessageBox( msg, _( "Library Load Error" ),
+                          wxOK | wxICON_ERROR, this );
+            continue;
+        }
+
+        msg.Printf( _( "Scan Lib: %s" ), GetChars( tmp ) );
         Affiche_Message( msg );
 
-        /* lecture entete chaine definie par ENTETE_LIBRAIRIE */
+        /* Reading header ENTETE_LIBRAIRIE */
         LineNum = 0;
-        GetLine( lib_module, Line, &LineNum );
+        GetLine( file, Line, &LineNum );
         StrPurge( Line );
         if( strnicmp( Line, ENTETE_LIBRAIRIE, L_ENTETE_LIB ) != 0 )
         {
-            DisplayError( winaff, _( "File is not a library" ) );
+            msg.Printf( _( "<%s> is not a valid Kicad PCB footprint library file." ),
+                        GetChars( tmp ) );
+            wxMessageBox( msg, _( "Library Load Error" ),
+                          wxOK | wxICON_ERROR, this );
+            fclose( file );
             return NULL;
         }
 
-        /* Lecture de la liste des composants de la librairie */
+        /* Reading the list of modules in the library. */
         Found = 0;
-        while( !Found && GetLine( lib_module, Line, &LineNum ) )
+        while( !Found && GetLine( file, Line, &LineNum ) )
         {
             if( strnicmp( Line, "$MODULE", 6 ) == 0 )
                 break;
             if( strnicmp( Line, "$INDEX", 6 ) == 0 )
             {
-                while( GetLine( lib_module, Line, &LineNum ) )
+                while( GetLine( file, Line, &LineNum ) )
                 {
                     if( strnicmp( Line, "$EndINDEX", 9 ) == 0 )
                         break;
                     StrPurge( Line );
                     msg = CONV_FROM_UTF8( Line );
-                    if( msg.CmpNoCase( ComponentName ) == 0 )
+                    if( msg.CmpNoCase( aModuleName ) == 0 )
                     {
-                        Found = 1; break; /* Trouve! */
+                        Found = 1;
+                        break; /* Found! */
                     }
                 }
             }
         }
 
-        /* Lecture de la librairie */
-        while( Found && GetLine( lib_module, Line, &LineNum ) )
+        /* Read library. */
+        while( Found && GetLine( file, Line, &LineNum ) )
         {
             if( Line[0] != '$' )
                 continue;
@@ -255,117 +297,136 @@ MODULE* WinEDA_BasePcbFrame::Get_Librairie_Module( wxWindow* winaff,
                 continue;
             if( strnicmp( Line, "$MODULE", 7 ) != 0 )
                 continue;
-            /* Lecture du nom du composant */
+            /* Read module name. */
             Name = CONV_FROM_UTF8( Line + 8 );
-            if( Name.CmpNoCase( ComponentName ) == 0 )  /* composant localise */
+
+            if( Name.CmpNoCase( aModuleName ) == 0 )
             {
                 NewModule = new MODULE( GetBoard() );
 
-                // Switch the locale to standard C (needed to print floating point numbers like 1.3)
-                SetLocaleTo_C_standard( );
-                NewModule->ReadDescr( lib_module, &LineNum );
-                SetLocaleTo_Default( );        // revert to the current  locale
-
+                // Switch the locale to standard C (needed to print
+                // floating point numbers like 1.3)
+                SetLocaleTo_C_standard();
+                NewModule->ReadDescr( file, &LineNum );
+                SetLocaleTo_Default();         // revert to the current locale
                 GetBoard()->Add( NewModule, ADD_APPEND );
-                fclose( lib_module );
+                fclose( file );
                 Affiche_Message( wxEmptyString );
                 return NewModule;
             }
         }
 
-        fclose( lib_module ); lib_module = 0;
+        fclose( file );
+        if( one_lib )
+            break;
     }
 
-    if( lib_module )
-        fclose( lib_module );
-
-    if( show_msg_err )
+    if( aDisplayMessageError )
     {
-        msg.Printf( _( "Module <%s> not found" ), ComponentName.GetData() );
-        DisplayError( winaff, msg );
+        msg.Printf( _( "Module <%s> not found" ), GetChars( aModuleName ) );
+        DisplayError( NULL, msg );
     }
+
     return NULL;
 }
 
 
-/***************************************************************/
+/** Function Select_1_Module_From_List
+ *  Display a list of modules found in active libraries or a given library
+ *  @param aLibraryFullFilename = library to list (if aLibraryFullFilename ==
+ *                                void, list all modules)
+ *  @param aMask = Display filter (wildcard)( Mask = wxEmptyString if not used
+ * )
+ *  @param aKeyWord = keyword list, to display a filtered list of module having
+ *                    one (or more) of these keyworks in their keywork list
+ *                    ( aKeyWord = wxEmptyString if not used )
+ *
+ *  @return wxEmptyString if abort or fails, or the selected module name if Ok
+ */
 wxString WinEDA_BasePcbFrame::Select_1_Module_From_List(
     WinEDA_DrawFrame* active_window,
-    const wxString& Library,
-    const wxString& Mask, const wxString& KeyWord )
-/***************************************************************/
-
-/*
- *  Affiche la liste des modules des librairies
- *  Recherche dans la librairie Library ou generale si Library == NULL
- *  Mask = Filtre d'affichage( Mask = wxEmptyString pour listage non filtr )
- *  KeyWord = Liste de mots cles, Recherche limitee aux composants
- *      ayant ces mots cles ( KeyWord = wxEmptyString pour listage de tous les modules )
- *
- *  retourne wxEmptyString si abort ou probleme
- *  ou le nom du module
- */
+    const wxString& aLibraryFullFilename,
+    const wxString& aMask, const wxString& aKeyWord )
 {
     int             LineNum;
     unsigned        ii, NbModules;
     char            Line[1024];
-    wxString        FullLibName;
-    static wxString OldName;/* Memorise le nom du dernier composant charge */
-    wxString        CmpName;
-    FILE*           lib_module;
+    wxFileName      fn;
+    static wxString OldName;    /* Save the name of the last module loaded. */
+    wxString        CmpName, tmp;
+    FILE*           file;
     wxString        msg;
 
     WinEDAListBox*  ListBox = new WinEDAListBox( active_window, wxEmptyString,
-                                                NULL, OldName, DisplayCmpDoc,
-                                                wxColour( 200, 200, 255 ) );
+                                                 NULL, OldName, DisplayCmpDoc,
+                                                 wxColour( 200, 200, 255 ),
+                                                 GetComponentDialogPosition() );
 
     wxBeginBusyCursor();
 
-    /* Recherche des composants en librairies */
+    /* Find modules in libraries. */
     NbModules = 0;
     for( ii = 0; ii < g_LibName_List.GetCount(); ii++ )
     {
-        /* Calcul du nom complet de la librairie */
-        if( Library.IsEmpty() )
+        /* Calculate the full file name of the library. */
+        if( aLibraryFullFilename.IsEmpty() )
         {
-            FullLibName = MakeFileName( g_RealLibDirBuffer,
-                                        g_LibName_List[ii], LibExtBuffer );
+            fn = wxFileName( wxEmptyString, g_LibName_List[ii],
+                             ModuleFileExtension );
         }
         else
-            FullLibName = MakeFileName( g_RealLibDirBuffer, Library, LibExtBuffer );
+            fn = aLibraryFullFilename;
 
-        ReadDocLib( FullLibName );
+        tmp = wxGetApp().FindLibraryPath( fn );
 
-        if( !KeyWord.IsEmpty() )    /* Inutile de lire la librairie si selection
-                                     *  par mots cles, deja lus */
+        if( !tmp )
         {
-            if( !Library.IsEmpty() )
+            msg.Printf( _( "PCB footprint library file <%s> not found in search paths." ),
+                        GetChars( fn.GetFullName() ) );
+            wxMessageBox( msg, _( "Library Load Error" ),
+                          wxOK | wxICON_ERROR, this );
+            continue;
+        }
+
+        ReadDocLib( tmp );
+
+        if( !aKeyWord.IsEmpty() ) /* Don't read the library if selection
+                                   * by keywords, already read. */
+        {
+            if( !aLibraryFullFilename.IsEmpty() )
                 break;
             continue;
         }
 
-        if( ( lib_module = wxFopen( FullLibName, wxT( "rt" ) ) )  == NULL )
+        file = wxFopen( tmp, wxT( "rt" ) );
+
+        if( file  == NULL )
         {
-            if( !Library.IsEmpty() )
+            if( !aLibraryFullFilename.IsEmpty() )
                 break;
             continue;
         }
 
         // Statusbar library loaded message
-        msg = _( "Library " ); msg << FullLibName; msg << _(" loaded");
+        msg = _( "Library " ) + fn.GetFullPath() + _( " loaded" );
         Affiche_Message( msg );
 
-        /* lecture entete */
+        /* Read header. */
         LineNum = 0;
-        GetLine( lib_module, Line, &LineNum, sizeof(Line) - 1 );
+        GetLine( file, Line, &LineNum, sizeof(Line) - 1 );
 
         if( strnicmp( Line, ENTETE_LIBRAIRIE, L_ENTETE_LIB ) != 0 )
         {
-            DisplayError( this, wxT( "This file is not an Eeschema libray file" ), 20 ); continue;
+            msg.Printf( _( "<%s> is not a valid Kicad PCB footprint library file." ),
+                        GetChars( tmp ) );
+            wxMessageBox( msg, _( "Library Load Error" ),
+                          wxOK | wxICON_ERROR, this );
+            fclose( file );
+            continue;
         }
 
-        /* Lecture de la librairie */
-        while( GetLine( lib_module, Line, &LineNum, sizeof(Line) - 1 ) )
+        /* Read library. */
+        while( GetLine( file, Line, &LineNum, sizeof(Line) - 1 ) )
         {
             if( Line[0] != '$' )
                 continue;
@@ -373,40 +434,41 @@ wxString WinEDA_BasePcbFrame::Select_1_Module_From_List(
                 break;
             if( strnicmp( Line, "$INDEX", 6 ) == 0 )
             {
-                while( GetLine( lib_module, Line, &LineNum ) )
+                while( GetLine( file, Line, &LineNum ) )
                 {
                     if( strnicmp( Line, "$EndINDEX", 9 ) == 0 )
                         break;
                     strupper( Line );
                     msg = CONV_FROM_UTF8( StrPurge( Line ) );
-                    if( Mask.IsEmpty() )
+                    if( aMask.IsEmpty() )
                     {
                         ListBox->Append( msg );
                         NbModules++;
                     }
-                    else if( WildCompareString( Mask, msg, FALSE ) )
+                    else if( WildCompareString( aMask, msg, FALSE ) )
                     {
                         ListBox->Append( msg );
                         NbModules++;
                     }
                 }
-            } /* Fin Lecture INDEX */
+            } /* End read INDEX */
         }
 
-        /* Fin lecture 1 Librairie */
+        /* End read library. */
+        fclose( file );
+        file = NULL;
 
-        fclose( lib_module ); lib_module = NULL;
-        if( !Library.IsEmpty() )
+        if( !aLibraryFullFilename.IsEmpty() )
             break;
     }
 
-    /*  creation de la liste des modules si recherche par mots-cles */
-    if( !KeyWord.IsEmpty() )
+    /* Create list of modules if search by keyword. */
+    if( !aKeyWord.IsEmpty() )
     {
         ModList* ItemMod = MList;
         while( ItemMod != NULL )
         {
-            if( KeyWordOk( KeyWord, ItemMod->m_KeyWord ) )
+            if( KeyWordOk( aKeyWord, ItemMod->m_KeyWord ) )
             {
                 NbModules++;
                 ListBox->Append( ItemMod->m_Name );
@@ -429,7 +491,6 @@ wxString WinEDA_BasePcbFrame::Select_1_Module_From_List(
 
     ListBox->Destroy();
 
-    /* liberation mem de la liste des textes doc module */
     while( MList != NULL )
     {
         ModList* NewMod = MList->Next;
@@ -444,25 +505,22 @@ wxString WinEDA_BasePcbFrame::Select_1_Module_From_List(
 }
 
 
-/******************************************/
-static void DisplayCmpDoc( wxString& Name )
-/*******************************************/
-
-/* Routine de recherche et d'affichage de la doc du composant Name
- *  La liste des doc est pointee par MList
+/* Find and display the doc Component Name
+ * The list of doc is pointed to by mlist.
  */
+static void DisplayCmpDoc( wxString& Name )
 {
     ModList* Mod = MList;
 
     if( !Mod )
     {
-        Name.Empty(); return;
+        Name.Empty();
+        return;
     }
 
-    /* Recherche de la description */
     while( Mod )
     {
-        if( !Mod->m_Name.IsEmpty() && (Mod->m_Name.CmpNoCase( Name ) == 0) )
+        if( !Mod->m_Name.IsEmpty() && ( Mod->m_Name.CmpNoCase( Name ) == 0 ) )
             break;
         Mod = Mod->Next;
     }
@@ -478,36 +536,32 @@ static void DisplayCmpDoc( wxString& Name )
 }
 
 
-/***************************************************/
-static void ReadDocLib( const wxString& ModLibName )
-/***************************************************/
-
-/* Routine de lecture du fichier Doc associe a la librairie ModLibName.
- *  Cree en memoire la chaine liste des docs pointee par MList
- *  ModLibName = full file Name de la librairie Modules
+/* Read the doc file and combine with a library ModLibName.
+ * Load in memory the list of docs string pointed to by mlist
+ * ModLibName = full file name of the library modules
  */
+static void ReadDocLib( const wxString& ModLibName )
 {
-    ModList* NewMod;
-    char     Line[1024];
-    FILE*    LibDoc;
-    wxString FullModLibName = ModLibName;
+    ModList*   NewMod;
+    char       Line[1024];
+    FILE*      LibDoc;
+    wxFileName fn = ModLibName;
 
-    ChangeFileNameExt( FullModLibName, EXT_DOC );
+    fn.SetExt( EXT_DOC );
 
-    if( ( LibDoc = wxFopen( FullModLibName, wxT( "rt" ) ) ) == NULL )
+    if( ( LibDoc = wxFopen( fn.GetFullPath(), wxT( "rt" ) ) ) == NULL )
         return;
 
     GetLine( LibDoc, Line, NULL, sizeof(Line) - 1 );
     if( strnicmp( Line, ENTETE_LIBDOC, L_ENTETE_LIB ) != 0 )
         return;
 
-    /* Lecture de la librairie */
     while( GetLine( LibDoc, Line, NULL, sizeof(Line) - 1 ) )
     {
         if( Line[0] != '$' )
             continue;
         if( Line[1] == 'E' )
-            break;;
+            break; ;
         if( Line[1] == 'M' ) /* Debut decription 1 module */
         {
             NewMod = new ModList();
@@ -533,33 +587,28 @@ static void ReadDocLib( const wxString& ModLibName )
                     break;
                 }
             }
-        } /* lecture 1 descr module */
+        } /* End read 1 module. */
     }
 
-    /* Fin lecture librairie */
     fclose( LibDoc );
 }
 
 
-/********************************************************************/
-MODULE* WinEDA_BasePcbFrame::Select_1_Module_From_BOARD( BOARD* Pcb )
-/********************************************************************/
-
-/* Affiche la liste des modules du PCB en cours
- *  Retourne un pointeur si module selectionne
- *  retourne NULL sinon
+/* Display the list of modules currently PCB
+ * Returns a pointer if module selected
+ * Returns NULL otherwise
  */
+MODULE* WinEDA_BasePcbFrame::Select_1_Module_From_BOARD( BOARD* Pcb )
 {
     int             ii;
     MODULE*         Module;
-    static wxString OldName;/* Memorise le nom du dernier composant charge */
+    static wxString OldName;       /* Save name of last module selectec. */
     wxString        CmpName, msg;
 
     WinEDAListBox*  ListBox = new WinEDAListBox( this, wxEmptyString,
-                                                NULL, wxEmptyString, NULL,
-                                                wxColour( 200, 200, 255 ) );
+                                                 NULL, wxEmptyString, NULL,
+                                                 wxColour( 200, 200, 255 ) );
 
-    /* Recherche des composants en BOARD */
     ii     = 0;
     Module = Pcb->m_Modules;
     for( ; Module != NULL; Module = (MODULE*) Module->Next() )
@@ -586,7 +635,6 @@ MODULE* WinEDA_BasePcbFrame::Select_1_Module_From_BOARD( BOARD* Pcb )
 
     OldName = CmpName;
 
-    // Recherche du pointeur sur le module
     Module = Pcb->m_Modules;
     for( ; Module != NULL; Module = (MODULE*) Module->Next() )
     {
