@@ -5,7 +5,7 @@
  
     Licence: see kboollicense.txt 
  
-    RCS-ID: $Id: scanbeam.h,v 1.4 2008/09/05 19:01:14 titato Exp $
+    RCS-ID: $Id: scanbeam.h,v 1.6 2009/09/10 17:04:09 titato Exp $
 */
 
 #ifndef SCANBEAM_H
@@ -20,10 +20,10 @@
 enum SCANTYPE{NODELINK, LINKLINK, GENLR, LINKHOLES, INOUT};
 
 #if defined(WXART2D_USINGDLL)
-template class A2DKBOOLDLLEXP DL_Iter<Record*>;
+template class A2DKBOOLDLLEXP DL_Iter<kbRecord*>;
 #endif
 
-class A2DKBOOLDLLEXP ScanBeam : public DL_List<Record*>
+class A2DKBOOLDLLEXP ScanBeam : public DL_List<kbRecord*>
 {
 protected:
     Bool_Engine* _GC;
@@ -31,26 +31,26 @@ protected:
 public:
     ScanBeam( Bool_Engine* GC );
     ~ScanBeam();
-    void SetType( Node* low, Node* high );
+    void SetType( kbNode* low, kbNode* high );
 
-    bool  FindNew( SCANTYPE scantype, TDLI<KBoolLink>* _I, bool& holes );
-    bool  RemoveOld( SCANTYPE scantype, TDLI<KBoolLink>* _I, bool& holes );
+    bool  FindNew( SCANTYPE scantype, TDLI<kbLink>* _I, bool& holes );
+    bool  RemoveOld( SCANTYPE scantype, TDLI<kbLink>* _I, bool& holes );
 
 private:
 
-    bool  ProcessHoles( bool atinsert, TDLI<KBoolLink>* _LI );
+    bool  ProcessHoles( bool atinsert, TDLI<kbLink>* _LI );
     int Process_LinkToLink_Crossings();   // find crossings
     int  Process_PointToLink_Crossings();
-    int  Process_LinkToLink_Flat( KBoolLine* flatline );
+    int  Process_LinkToLink_Flat( kbLine* flatline );
     void SortTheBeam( bool backangle );
     bool  checksort();
     bool  writebeam();
     void Calc_Ysp();
-    //int  FindCloseLinksAndCross(TDLI<KBoolLink>* _I,Node* _lowf);
+    //int  FindCloseLinksAndCross(TDLI<kbLink>* _I,kbNode* _lowf);
     void  Generate_INOUT( int graphnumber );
 
-    Node*      _low;
-    DL_Iter<Record*> _BI;
+    kbNode*      _low;
+    DL_Iter<kbRecord*> _BI;
     int       lastinserted;
     BEAM_TYPE    _type;
 };
